; FDA - Frequency domain analyzer
;	Processor : PIC18F4520 4.0MHz*4 clock
;	LCD : S6B0107/0108 type graphics module
;	DDS : AD9834BURZ, 50.0MHz clock
;	Copyright(C) 2010 K.Wakizawa

	processor	PIC18F4520
	include		p18f4520.inc

; configuration bits
	CONFIG	OSC = HSPLL	;HS oscillator+PLL(x4)
	CONFIG	FCMEN = OFF	;fail safe clock - not use
	CONFIG	IESO = OFF	;internal/external swichover - not use
	CONFIG	PWRT = OFF	;Powerup timer - not use
	CONFIG	BOREN = ON	;brownout detect - ON
	CONFIG	BORV = 2		;brownout voltage - 2.7V
	CONFIG	WDT = ON		;watchdog timer enable
	CONFIG	WDTPS = 1024	;watchdog tomer prescaler 1:1024 abouut 4 Sec.
	CONFIG	MCLRE = ON	;MCLR pin enable - external reset
	CONFIG	LPT1OSC = OFF	;low power timer1 osc. - not use
	CONFIG	PBADEN = OFF	;port B ADC - not use
	CONFIG	CCP2MX = PORTC	;CCP2 I/O is multiplexed with RC1
	CONFIG	STVREN = ON	;Stack overflow reset - enable
	CONFIG	LVP = OFF	;low voltage program - disable
	CONFIG	XINST = OFF	;external instruction - not use
	CONFIG	DEBUG = OFF	;release mode
; protection configuration
	CONFIG	CP0 = OFF	;code area protections - disable
	CONFIG	CP1 = OFF
	CONFIG	CP2 = OFF
	CONFIG	CP3 = OFF
	CONFIG	CPB = OFF
	CONFIG	CPD = OFF
	CONFIG	WRT0 = OFF	;Flash memory write protections - disable
	CONFIG	WRT1 = OFF
	CONFIG	WRT2 = OFF
	CONFIG	WRT3 = OFF
	CONFIG	WRTB = OFF
	CONFIG	WRTC = OFF
	CONFIG	WRTD = OFF
	CONFIG	EBTR0 = OFF	;Table read protections - disable
	CONFIG	EBTR1 = OFF
	CONFIG	EBTR2 = OFF
	CONFIG	EBTR3 = OFF
	CONFIG	EBTRB = OFF



WENB	macro
	goto	$+2
	goto	$+2
	endm
;
; I/O definition
; PORTA - analog input
LVL	equ	0	;(AIN)signal level(low gain)
LVH	equ	1	;(AIN)signal leve;(high gain)
PHA	equ	2	;(AIN)from phase detector
VBT	equ	3	;(AIN)battery voltage
;   RS4 and RA5 are not use
; PORTB - LCD control and other I/O
LCSL	equ	0	;(OUT)LCD left side select
LCSR	equ	1	;(OUT)LCD right side select
LE	equ	2	;(OUT)LCD enable
LRW	equ	3	;(OUT)LCD R/W
LRS	equ	4	;(OUT)LCD RS
REP	equ	5	;(IN)rotary encoder push switch
;   RB6, RB7 are ICSP/ICE pin
; PORTC - USART and other I/O
REB	equ	0	;(IN)rotary encoder phase B
REA	equ	1	;(IN)rotary encoder phase A
REABIT	equ	b'00000010'
DRE	equ	2	;(OUT)DDS RESET
PSW3	equ	3	;(IN)push switch #3(ON/OFF)
PSW2	equ	4	;(IN)push switch @2(SELECT)
PSW1	equ	5	;(IN)push switch #1(MODE)
TX	equ	6	;(USART)TX data
RX	equ	7	;(USART)RX data
; PORTD - LCD data
LCDDATA	equ	PORTD
; PORTE - DDS control
DSY	equ	0	;(OUT)DDS flame sync
DDA	equ	1	;(OUT)DDS serial data
DCK	equ	2	;(OUT)DDS serial clock

; work area -------------------------
TMP0	equ	h'010'		;temporary work area
TMP1	equ	h'011'
TMP2	equ	h'012'
TMP3	equ	h'013'
ACC0_W	equ	h'018'
ACC1_W	equ	h'01A'
ACC2_D	equ	h'01C'		;32 bit accum.
A48	equ	h'020'		;48 bit accum.

RESAVE	equ	h'026'		;rotary encoderwork
RECOUNT	equ	h'027'		;rotary encoder count(+127..-128)
	
RXRPTR	equ	h'028'		;COM receive data pointer
RXWPTR	equ	h'029'		;and counter
RXCNT	equ	h'02A'
LASTCHR	equ	h'02B'		;last character
;
FREQ_D	equ	h'02C'		;target freq.
START_D	equ	h'030'		;sweep start ferq.
STEP_W	equ	h'034'		;freq.step
MOD_W	equ	h'036'		;modulation freq.
DEV_W	equ	h'038'		;freq. deviation(FSK)
GAIN_W	equ	h'03A'		;sweep mode detector gain
FUNCTION	equ	h'03C'		;function #
OUTPUT	equ	h'03D'		;output enable flag
MODEFY	equ	h'03E'		;data modefy flag
MODTYP	equ	h'03F'		;modu.type(0:CW, 1:OOK, 2:FSK)
WAVEFM	equ	h'040'		;waveform select(0:sin, 1:triangle)
FR_GA	equ	h'040'		;dial select(0:freq, 1:gain)
GA_PH	equ	h'41'		;display mode(1:gain, 2:phase, 3:gain+phase)
STEPPTR	equ	h'042'		;freq.step pointer(for TFSTEP,LOW)

PX	equ	h'043'		;X pos
PY	equ	h'044'		;Y pos

MODCTL_D	equ	h'46'		;moduration control reg.
T1TMG_W	equ	h'4A'		;TMR1 interrupt timing
T1FLAG	equ	h'4C'		;TMR1 event flag

OFSETH	equ	h'4E'		;high level detector offset
OFSETL	equ	h'4F'		;low level detector offset

; string buffer
SBUF	equ	h'50'		;string buffer(12 byte)

; context save area
F_SAVE	equ	h'05C'
S_SAVE	equ	h'05E'
W_SAVE	equ	h'05F'


RXBUF	equ	h'0C0'		;COM port receive buffer 64 bytes

; P R O G R A M   S T A R T S   H E R E =============
; start vector/interrupt vector
	org	h'0000'		;RESET
	clrf	INTCON
	goto	START
	org	h'0008'		;TMR0 interrupt
	goto	INTERRUPT_HIGH
	org	h'0018'
	goto	INTERRUPT_LOW	;USART_RX and TMR1 interrupt
; program code
START:
	call	INIZ_PORT	;I/O port direction and startup iniz.
	call	INIZ_COM		;USART iniz.
	call	INIZ_TMR0	;TMR0 interrupt iniz(interval timer) 
	call	INIZ_LCD		;LCD display iniz.
; file reg. iniz.
	movlw	LOW  d'1000'	;freq.step is 1KHz
	movwf	STEP_W
	movlw	HIGH d'1000'
	movwf	STEP_W+1
	movlw	d'9'
	movwf	STEPPTR
	movlw	LOW d'1000000'	;DDS output freq.is 1MHz
	movwf	FREQ_D
	movwf	START_D		;scan start freq.is 1MHz
	movlw	HIGH d'1000000'
	movwf	FREQ_D+1
	movwf	START_D+1
	movlw	UPPER d'1000000'
	movwf	FREQ_D+2
	movwf	START_D+2
	clrf	FREQ_D+3
	clrf	START_D+3
	movlw	LOW d'1000'	;modulation freq.is 1KHz
	movwf	MOD_W
	movlw	HIGH d'1000'
	movwf	MOD_W+1
	movlw	LOW d'10000'	;modulation deviation is 10KHz
	movwf	DEV_W
	movlw	HIGH d'10000'
	movwf	DEV_W+1
	movlw	d'30'		;level display gain
	movwf	GAIN_W
	clrf	GAIN_W+1
	clrf	MODEFY		;clear modefy flag
	clrf	MODTYP		;clear modulation type(CW)
	clrf	OUTPUT		;output signal off
	clrf	FUNCTION		;function select is 0; stop DDS
	clrf	FR_GA
	movlw	d'3'
	movwf	GA_PH
;
; SG mode - Signal Generator ===========================================
SG_MODE:
	call	LCD_CLR		;clear screen
	clrf	PX
	clrf	PY
	movlw	LOW M_MODE_SG	;put [SG Mode]
	movwf	ACC0_W
	movlw	HIGH M_MODE_SG
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
; display frequency(MAIN)
SGM1:	movlw	d'8'		;put [Freq.]
	movwf	PX
	movlw	d'1'
	movwf	PY
	movlw	LOW M_MSG_FREQ
	movwf	ACC0_W
	movlw	HIGH M_MSG_FREQ
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'		;put current freq.
	movwf	PX
	call	DISP_FREQ
; change step
	movlw	d'8'		;put [Step]
	movwf	PX
	movlw	d'2'
	movwf	PY
	movlw	LOW M_MSG_STEP
	movwf	ACC0_W
	movlw	HIGH M_MSG_STEP
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'		;put freq.step
	movwf	PX
	movff	STEP_W,ACC0_W
	movff	STEP_W+1,ACC0_W+1
	call	DISP_STEP
; sisplay waveform
	movlw	d'8'		;put [Wave]
	movwf	PX
	movlw	d'3'
	movwf	PY
	movlw	LOW M_MSG_WAVE
	movwf	ACC0_W
	movlw	HIGH M_MSG_WAVE
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	movf	WAVEFM,w		;check waveform
	BNZ	SGM11
	movlw	LOW M_WAVE_SIN	;put [Sin]
	movwf	ACC0_W
	movlw	HIGH M_WAVE_SIN
	movwf	ACC0_W+1
	goto	SGM12
SGM11:	movlw	LOW M_WAVE_TRI	;put [Tri]
	movwf	ACC0_W
	movlw	HIGH M_WAVE_TRI
	movwf	ACC0_W+1
SGM12:	call	PSTRING_LCD_FLASH
; display modulation
	movlw	d'8'		;put [Modu]
	movwf	PX
	movlw	d'4'
	movwf	PY
	movlw	LOW M_MSG_MOD
	movwf	ACC0_W
	movlw	HIGH M_MSG_MOD
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	movf	MODTYP,w		;check modu.type
	BNZ	SGM15
	movlw	LOW M_MOD_CW	;put [CW]
	movwf	ACC0_W
	movlw	HIGH M_MOD_CW
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	goto	SGM17
SGM15:	btfsc	MODTYP,1		;check modu.type(OOK or FSK)
	goto	SGM16
	movlw	LOW M_MOD_OOK	;OOK mode - put [OOK]
	movwf	ACC0_W
	movlw	HIGH M_MOD_OOK
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'8'
	movwf	PX
	movlw	d'5'
	movwf	PY
	movlw	LOW M_MSG_MDFR	;put [MdFr]
	movwf	ACC0_W
	movlw	HIGH M_MSG_MDFR
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'		;put modulation freq.
	movwf	PX
	movff	MOD_W,ACC0_W
	movff	MOD_W+1,ACC0_W+1
	call	DISP_STEP
	goto	SGM17
SGM16:	movlw	LOW M_MOD_FSK	;FSK mode - put [FSK]
	movwf	ACC0_W
	movlw	HIGH M_MOD_FSK
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'8'
	movwf	PX
	movlw	d'5'
	movwf	PY
	movlw	LOW M_MSG_MDFR	;put [MdFr]
	movwf	ACC0_W
	movlw	HIGH M_MSG_MDFR
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'		;put modulation freq.
	movwf	PX
	movff	MOD_W,ACC0_W
	movff	MOD_W+1,ACC0_W+1
	call	DISP_STEP
	movlw	d'8'		;put [Dev]
	movwf	PX
	movlw	d'6'
	movwf	PY
	movlw	LOW M_MSG_DEV
	movwf	ACC0_W
	movlw	HIGH M_MSG_DEV
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'		;put freq.Deviation
	movwf	PX
	movff	DEV_W,ACC0_W
	movff	DEV_W+1,ACC0_W+1
	call	DISP_STEP
; output status
SGM17:	movlw	d'8'		;put [Out]
	movwf	PX
	movlw	d'7'
	movwf	PY
	movlw	LOW M_MSG_OUT
	movwf	ACC0_W
	movlw	HIGH M_MSG_OUT
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	movf	OUTPUT,w		;check output status
	BNZ	SGM18
	movlw	LOW M_MSG_OFF	;put [Off]
	movwf	ACC0_W
	movlw	HIGH M_MSG_OFF
	movwf	ACC0_W+1
	goto	SGM19
SGM18:	movlw	LOW M_MSG_ON	;put [ON]
	movwf	ACC0_W
	movlw	HIGH M_MSG_ON
	movwf	ACC0_W+1
SGM19:	call	PSTRING_LCD_FLASH
SGM1A:	clrwdt			;wait for switch release
	movff	PORTC,TMP0	
	movlw	b'11000111'
	iorwf	TMP0,f
	incf	TMP0,w
	BNZ	SGM1A
	incf	FUNCTION,w	;(pos.adjust)
	movwf	PY
	clrf	PX
	movlw	'>'
	call	PUTC_LCD
; check input change and jump to indivisual routine
SGM2:	clrwdt
#if 0
	clrf	PX
	movlw	d'5'
	movwf	PY
	movlw	0
	call	ADC
	movff	ACC0_W,ACC2_D
	movff	ACC0_W+1,ACC2_D+1
	clrf	ACC2_D+2
	clrf	ACC2_D+3
	call	PUTDEC
	call	PSTRING_LCD
	clrf	PX
	movlw	d'6'
	movwf	PY
	movlw	1
	call	ADC
	movff	ACC0_W,ACC2_D
	movff	ACC0_W+1,ACC2_D+1
	clrf	ACC2_D+2
	clrf	ACC2_D+3
	call	PUTDEC
	call	PSTRING_LCD
	call	PCRLF
#endif

; check switches --------------
	movff	PORTC,TMP0
	movlw	b'11000111'
	iorwf	TMP0,f
	incf	TMP0,w
	BZ	SGM3		;not pressed
	btfss	TMP0,PSW1
	goto	SGM_MODE_NEXT
	btfss	TMP0,PSW2
	goto	SGM25
; PSW3 pressed.
; output ON/OFF control
	movf	OUTPUT,w
	BZ	SGM21
; output turn off
	clrf	OUTPUT
	goto	SG_SET2DDS
; output turn on
SGM21:	movlw	h'ff'
	movwf	OUTPUT
	goto	SG_SET2DDS
; function select
SGM25:	incf	FUNCTION,f
	movf	MODTYP,w
	BZ	SGM27		;CW mode - no additional display
	decf	MODTYP,w
	BZ	SGM26
	movlw	6		;FSK mode - max 6 lines
	goto	SGM28
SGM26:	movlw	5		;OOK mode - max 5 lines
	goto	SGM28
SGM27:	movlw	4		;CW mode - max 4 lines
SGM28:	subwf	FUNCTION,w
	btfsc	STATUS,Z
	clrf	FUNCTION
	movlw	d'10'		;wait 10mS and return
	call	mSleep
	goto	SG_MODE
; check rotary encoder ----------------
SGM3:	movf	RECOUNT,w
	BNZ	SGM31
	goto	SGM4		;rotary encoder not changed
SGM31:	movwf	TMP0		;TMP0 is rotary encoder data
	clrf	RECOUNT
	movff	FUNCTION,TMP1	;goto indivisual routines
	movf	TMP1,w		;0:Freq.
	BZ	SGM32
	decf	TMP1,f		;1:step
	BZ	SGM33
	decf	TMP1,f		;2:Waveform
	BZ	SGM34
	decf	TMP1,f		;3:modulation
	BZ	SGM35
	decf	TMP1,f		;4:mod.freq.
	BZ	SGM36
	goto	SGM37		;5:deviation
; change freq.
SGM32:	movff	TMP0,ACC0_W	;(ACC0_W) = ABS(TMP0)
	clrf	ACC0_W+1
	comf	TMP0,w
	addlw	1
	btfsc	ACC0_W,7		;check sign(when plus, slip nest instruction)
	movwf	ACC0_W
	movff	STEP_W,ACC1_W	;(ACC2_D) = (STWP_W) * ( ABS(RE data) )
	movff	STEP_W+1,ACC1_W+1
	call	MUL1616
	btfss	TMP0,7		;check sign
	goto	SGM325
	comf	ACC2_D,f		;RE data is minus
	comf	ACC2_D+1,f	;(ACC2_D) = 0 - (ACC2_D)
	comf	ACC2_D+2,f
	comf	ACC2_D+3,f
	incf	ACC2_D,f
	btfsc	STATUS,Z
	incf	ACC2_D+1,f
	btfsc	STATUS,Z
	incf	ACC2_D+2,f
	btfsc	STATUS,Z
	incf	ACC2_D+3,f
SGM325:	movf	ACC2_D,w		;(FREQ_D) += (ACC2_D)
	addwf	FREQ_D,f
	movf	ACC2_D+1,w
	addwfc	FREQ_D+1,f
	movf	ACC2_D+2,w
	addwfc	FREQ_D+2,f
	movf	ACC2_D+3,w
	addwfc	FREQ_D+3,f
; check minimum
	btfsc	FREQ_D+3,7	;check overflow
	goto	SGM326
	movf	FREQ_D+3,w
	BNZ	SGM327
	movf	FREQ_D+2,w
	BNZ	SGM327
	movf	FREQ_D+1,w
	BNZ	SGM327
	movlw	d'20'
	subwf	FREQ_D,w
	BC	SGM327
SGM326:	movlw	d'20'		;minumum freq. is 20Hz
	movwf	FREQ_D
	clrf	FREQ_D+1
	clrf	FREQ_D+2
	clrf	FREQ_D+3
; check maximum
SGM327:	movlw	LOW d'15000000'	;maximum freq. is 15MHz
	movwf	ACC2_D		;(15MHz) - (FREQ_D)
	movlw	HIGH d'15000000'
	movwf	ACC2_D+1
	movlw	UPPER d'15000000'
	movwf	ACC2_D+2
	clrf	ACC2_D+3
	movf	FREQ_D,w
	subwf	ACC2_D,f
	movf	FREQ_D+1,w
	subwfb	ACC2_D+1,f
	movf	FREQ_D+2,w
	subwfb	ACC2_D+2,f
	movf	FREQ_D+3,w
	subwfb	ACC2_D+3,f
	btfss	ACC2_D+3,7	;check overflow
	goto	SG_SET2DDS
	movlw	LOW d'15000000'	;(15MHz)
	movwf	FREQ_D
	movlw	HIGH d'15000000'
	movwf	FREQ_D+1
	movlw	UPPER d'15000000'
	movwf	FREQ_D+2
	clrf	FREQ_D+3
	goto	SG_SET2DDS
; change step
SGM33:	btfsc	TMP0,7		;check sign(direction)
	goto	SGM335
	incf	STEPPTR,f	;increase
SGM332:	movlw	b'00001111'
	andwf	STEPPTR,f
	call	SET_STEP		;STEPPTR to STEP_W
	goto	SG_SET2DDS
SGM335:	decf	STEPPTR,f	;decrease
	goto	SGM332
; change waveform
SGM34:	incf	WAVEFM,f		;change waveform flag
	movlw	b'00000001'
	andwf	WAVEFM,f
	goto	SG_SET2DDS
; change modulation
SGM35:	incf	MODTYP,f		
	movlw	d'3'
	subwf	MODTYP,w
	btfss	STATUS,Z
	goto	SG_SET2DDS
	clrf	MODTYP
	goto	SG_SET2DDS
; change mod.freq.
SGM36:	movff	TMP0,ACC0_W	;(ACC0_W) = ABS(TMP0)
	clrf	ACC0_W+1
	comf	TMP0,w
	addlw	1
	btfsc	TMP0,7		;check sign
	movwf	ACC0_W
	movlw	d'10'		;step 10Hz
	movwf	ACC1_W
	clrf	ACC1_W+1
	call	MUL1616
	btfss	TMP0,7		;check sign
	goto	SGM365
	comf	ACC2_D		;(ACC2_D) = 0 - (ACC2_D)
	comf	ACC2_D+1
	incf	ACC2_D,f
	btfsc	STATUS,Z
	incf	ACC2_D+1,f
SGM365:	movf	ACC2_D,w		;(MOD_W) += (ACC2_D)
	addwf	MOD_W,f
	movf	ACC2_D+1,w
	addwfc	MOD_W+1,f
	btfsc	MOD_W+1,7	;check underflow
	goto	SGM366
	movf	MOD_W,w		;check if zero
	iorwf	MOD_W+1,w
	BNZ	SGM367
SGM366:	movlw	d'10'		;minimum modulation freq. = 10Hz
	movwf	MOD_W
	clrf	MOD_W+1
SGM367:	movlw	LOW d'3000'	;maximum moduleation freq.= 3KHz
	movwf	ACC0_W		;check max
	movlw	HIGH d'3000'	
	movwf	ACC0_W+1
	movf	MOD_W,w
	subwf	ACC0_W,f
	movf	MOD_W+1,w
	subwfb	ACC0_W+1,f
	btfss	ACC0_W+1,7
	goto	SG_SET2DDS
	movlw	LOW d'3000'
	movwf	MOD_W
	movlw	HIGH d'3000'
	movwf	MOD_W+1
	goto	SG_SET2DDS
; change deviation
SGM37:	movff	TMP0,ACC0_W	;(ACC0_W) = ABS(TMP0)
	clrf	ACC0_W+1
	comf	TMP0,w
	addlw	1
	btfsc	TMP0,7		;check sign
	movwf	ACC0_W
	movlw	d'100'		;step 100Hz
	movwf	ACC1_W
	clrf	ACC1_W+1
	call	MUL1616
	btfss	TMP0,7		;check sign
	goto	SGM375
	comf	ACC2_D		;(ACC0_W) = 0 - (ACC0_W)
	comf	ACC2_D+1
	incf	ACC2_D,f
	btfsc	STATUS,Z
	incf	ACC2_D+1,f
SGM375:	movf	ACC2_D,w		;(DEV_W) += (ACC0_W)
	addwf	DEV_W,f
	movf	ACC2_D+1,w
	addwfc	DEV_W+1,f
	btfsc	DEV_W+1,7	;check underflow
	goto	SGM376
	movf	DEV_W,w		;check if zero
	iorwf	DEV_W+1,w
	BNZ	SGM377
SGM376:	movlw	d'100'		;minimum deviation is 100Hz
	movwf	DEV_W
	clrf	DEV_W+1
SGM377:	movlw	LOW d'30000'	;maximum deviation is 30KHz
	movwf	ACC0_W
	movlw	HIGH d'30000'
	movwf	ACC0_W+1
	movf	DEV_W,w
	subwf	ACC0_W,f
	movf	DEV_W+1,w
	subwfb	ACC0_W+1,f
	btfss	ACC0_W+1,7
	goto	SG_SET2DDS
	movlw	LOW d'30000'
	movwf	DEV_W
	movlw	HIGH d'30000'
	movwf	DEV_W+1
	goto	SG_SET2DDS

; check modulation interrupt
SGM4:	clrwdt
	btfss	T1FLAG,0
	goto	SGM2		;not change
	clrf	T1FLAG
; modulation - change control reg
	movff	MODCTL_D,ACC0_W
	movff	MODCTL_D+1,ACC0_W+1
	call	WRDDS
	movff	MODCTL_D,ACC0_W	;swap MODCTL_D lower and upper 16bit
	movff	MODCTL_D+1,ACC0_W+1
	movff	MODCTL_D+2,MODCTL_D
	movff	MODCTL_D+3,MODCTL_D+1
	movff	ACC0_W,MODCTL_D+2
	movff	ACC0_W+1,MODCTL_D+3
	goto	SGM2

SG_SET2DDS:
	clrf	ACC0_W		;stop TMR1 interrupt
	clrf	ACC0_W+1
	call	INIZ_TMR1
; stop DDS
; control reg.
	movlw	b'00100001'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......1	;RESET
	movwf	ACC0_W+1
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	ACC0_W
	call	WRDDS
; calc. and set frequency
	btfsc	MODTYP,1		;FSK mode?
	goto	S2DDS2
;  CW or OOK mode - one frequency
	movff	FREQ_D,ACC2_D	;calc. frequency
	movff	FREQ_D+1,ACC2_D+1
	movff	FREQ_D+2,ACC2_D+2
	movff	FREQ_D+3,ACC2_D+3
	call	CALC_FREQ

	movff	ACC2_D,ACC0_W	;set to DDS FREQ0 register
	movff	ACC2_D+1,ACC0_W+1
	bsf	ACC0_W+1,6
	call	WRDDS
	movff	ACC2_D+2,ACC0_W
	movff	ACC2_D+3,ACC0_W+1
	bsf	ACC0_W+1,6
	call	WRDDS

	movff	ACC2_D,ACC0_W	;set to DDS FREQ1 register
	movff	ACC2_D+1,ACC0_W+1
	bsf	ACC0_W+1,7
	call	WRDDS
	movff	ACC2_D+2,ACC0_W
	movff	ACC2_D+3,ACC0_W+1
	bsf	ACC0_W+1,7
	call	WRDDS
	goto	S2DDS3
;  FSK mode - two frequecy
S2DDS2:	movff	FREQ_D,ACC2_D	;calc. frequency +deviation
	movff	FREQ_D+1,ACC2_D+1
	movff	FREQ_D+2,ACC2_D+2
	movff	FREQ_D+3,ACC2_D+3
	movf	DEV_W,w
	addwf	ACC2_D,f
	movf	DEV_W+1,w
	addwfc	ACC2_D+1,f
	btfsc	STATUS,C
	incf	ACC2_D+2,f
	btfsc	STATUS,Z
	incf	ACC2_D+3,f
	call	CALC_FREQ

	movff	ACC2_D,ACC0_W	;set to DDS FREQ0 register
	movff	ACC2_D+1,ACC0_W+1
	bsf	ACC0_W+1,6
	call	WRDDS
	movff	ACC2_D+2,ACC0_W
	movff	ACC2_D+3,ACC0_W+1
	bsf	ACC0_W+1,6
	call	WRDDS

	movff	FREQ_D,ACC2_D	;calc. frequency -deviation
	movff	FREQ_D+1,ACC2_D+1
	movff	FREQ_D+2,ACC2_D+2
	movff	FREQ_D+3,ACC2_D+3
	movf	DEV_W,w
	subwf	ACC2_D,f
	movf	DEV_W+1,w
	subwfb	ACC2_D+1,f
	btfsc	STATUS,C
	decf	ACC2_D+2,f
	incf	ACC2_D+2,w
	btfsc	STATUS,Z
	decf	ACC2_D+3,f
	call	CALC_FREQ

	movff	ACC2_D,ACC0_W	;set to DDS FREQ1 register
	movff	ACC2_D+1,ACC0_W+1
	bsf	ACC0_W+1,7
	call	WRDDS
	movff	ACC2_D+2,ACC0_W
	movff	ACC2_D+3,ACC0_W+1
	bsf	ACC0_W+1,7
	call	WRDDS
; control reg.
S2DDS3:	movlw	b'00100000'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......0	;reset release
	movwf	ACC0_W+1
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	ACC0_W
;
	movf	OUTPUT,w
	btfsc	STATUS,Z
	bsf	ACC0_W+1,0	;output OFF(reset)
;
	movf	WAVEFM,w
	btfss	STATUS,Z
	bsf	ACC0_W,1		;triangle wave
	call	WRDDS
; check modulation
	movf	MODTYP,w
	BZ	S2DDS6		;CW
	decf	MODTYP,w
	BNZ	S2DDS4		;FSK
; OOK moduration data to ACC2_D
	movlw	b'00100000'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......0	;reset release
	movwf	MODCTL_D+1
	movwf	MODCTL_D+3
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	MODCTL_D
	movwf	MODCTL_D+2
;	
	bsf	MODCTL_D+1,0	;output OFF(reset)
;
	movf	WAVEFM,w
	btfss	STATUS,Z
	bsf	MODCTL_D,1		;triangle wave
	bsf	MODCTL_D+3,1
	goto	S2DDS5
; FSK modulation data to ACC2_D
S2DDS4:	movlw	b'00100000'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......0	;reset release
	movwf	MODCTL_D+1
	movwf	MODCTL_D+3
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	MODCTL_D
	movwf	MODCTL_D+2
;	
	bsf	MODCTL_D+1,3	;FREQ1 select bit
;
	movf	WAVEFM,w
	btfss	STATUS,Z
	bsf	MODCTL_D,1		;triangle wave
	bsf	MODCTL_D+2,1
;
S2DDS5:	movff	MOD_W,ACC0_W	;start modulation interrupt
	movff	MOD_W+1,ACC0_W+1
	call	INIZ_TMR1
; data set complete
S2DDS6:	movlw	d'10'
	call	mSleep
	goto	SG_MODE
; exit SG mode - change to freq.domain analysis mode
SGM_MODE_NEXT:
 	clrf	MODEFY
	clrf	MODTYP
	clrf	OUTPUT
	clrf	WAVEFM
	clrf	FUNCTION
; control reg. - stop DDS
	movlw	b'00100001'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......1	;RESET
	movwf	ACC0_W+1
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	ACC0_W
	call	WRDDS
;	goto	FANA_MODE

; Freq.Analysis Setting mode ==============================================
FASET_MODE:
	call	LCD_CLR		;clear screen
	clrf	PX
	clrf	PY
	movlw	LOW M_MODE_SWEEP
	movwf	ACC0_W
	movlw	HIGH M_MODE_SWEEP
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
; display start frequency
FAS1:	movlw	d'8'
	movwf	PX
	movlw	d'1'
	movwf	PY
	movlw	LOW M_MSG_START
	movwf	ACC0_W
	movlw	HIGH M_MSG_START
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	call	DISP_START
; change step
	movlw	d'8'
	movwf	PX
	movlw	d'2'
	movwf	PY
	movlw	LOW M_MSG_STEP
	movwf	ACC0_W
	movlw	HIGH M_MSG_STEP
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	movff	STEP_W,ACC0_W
	movff	STEP_W+1,ACC0_W+1
	call	DISP_STEP
; end (stop) freq.
	movlw	d'8'
	movwf	PX
	movlw	d'3'
	movwf	PY
	movlw	LOW M_MSG_END
	movwf	ACC0_W
	movlw	HIGH M_MSG_END
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	call	CALC_STOPF
	call	DFREQ1
; dial mode
	movlw	d'8'
	movwf	PX
	movlw	d'4'
	movwf	PY
	movlw	LOW M_MSG_DIAL
	movwf	ACC0_W
	movlw	HIGH M_MSG_DIAL
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	movf	FR_GA,w
	BNZ	FAS11
	movlw	LOW M_MSG_FREQ
	movwf	ACC0_W
	movlw	HIGH M_MSG_FREQ
	movwf	ACC0_W+1
	goto	FAS12
FAS11:	movlw	LOW M_MSG_GAIN
	movwf	ACC0_W
	movlw	HIGH M_MSG_GAIN
	movwf	ACC0_W+1
FAS12:	call	PSTRING_LCD_FLASH
; dsiplay mode
	movlw	d'8'
	movwf	PX
	movlw	d'5'
	movwf	PY
	movlw	LOW M_MSG_DISP
	movwf	ACC0_W
	movlw	HIGH M_MSG_DISP
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	d'38'
	movwf	PX
	btfss	GA_PH,0			;check gain display
	goto	FAS16
	movlw	LOW M_MSG_GAIN
	movwf	ACC0_W
	movlw	HIGH M_MSG_GAIN
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
FAS16:	btfss	GA_PH,1			;check phase display
	goto	FAS17
	movlw	LOW M_MSG_PHASE
	movwf	ACC0_W
	movlw	HIGH M_MSG_PHASE
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
FAS17:	movlw	d'3'
	subwf	GA_PH,w
	BZ	FAS18
	movlw	d'5'		;clear last display
	movwf	TMP3
FAS171:	movlw	' '	
	call	PUTC_LCD
	decfsz	TMP3,f
	goto	FAS171
;
FAS18:	clrwdt
	movff	PORTC,TMP0	;wait for switch release
	movlw	b'11000111'
	iorwf	TMP0,f
	incf	TMP0,w
	BNZ	FAS18
	incf	FUNCTION,w
	movwf	PY
	clrf	PX
	movlw	'>'
	call	PUTC_LCD
;
; Frequency analysis setting change
FAS2:	clrwdt
; check switches
	movff	PORTC,TMP0
	movlw	b'11000111'
	iorwf	TMP0,f
	incf	TMP0,w
	BZ	FAS3		;not press
	btfss	TMP0,PSW1
	goto	FAS_MODE_NEXT
	btfsc	TMP0,PSW2
; to Sweep and display mode
	goto	FAD_MODE
; function select
FAS25:	incf	FUNCTION,f
	movlw	5
	subwf	FUNCTION,w
	btfsc	STATUS,Z
	clrf	FUNCTION
	movlw	d'10'
	call	mSleep
	goto	FASET_MODE
; check rotary encoder
FAS3:	movf	RECOUNT,w
	BNZ	FAS31
	goto	FAS2
FAS31:	movwf	TMP0		;TMP0 is rotary encoder data
	clrf	RECOUNT
	movff	FUNCTION,TMP1
	movf	TMP1,w		;0:Start freq.
	BZ	FAS32
	decf	TMP1,f		;1:freq.Step
	BZ	FAS33
	decf	TMP1,f		;2:End freq.
	BZ	FAS32
	decf	TMP1,f		;3:Dial mode - frequency / gain
	BZ	FAS35
	decf	TMP1,f		;4:Display mode - gain / phase
	BZ	FAS36
; change start(end) freq. 
FAS32:	movff	TMP0,ACC0_W
	clrf	ACC0_W+1
	comf	TMP0,w
	addlw	1
	btfsc	ACC0_W,7		;check sign
	movwf	ACC0_W
	movff	STEP_W,ACC1_W
	movff	STEP_W+1,ACC1_W+1
	call	MUL1616
	btfss	TMP0,7		;check sign
	goto	FAS325
	comf	ACC2_D,f
	comf	ACC2_D+1,f
	comf	ACC2_D+2,f
	comf	ACC2_D+3,f
	incf	ACC2_D,f
	btfsc	STATUS,Z
	incf	ACC2_D+1,f
	btfsc	STATUS,Z
	incf	ACC2_D+2,f
	btfsc	STATUS,Z
	incf	ACC2_D+3,f
FAS325:	movf	ACC2_D,w
	addwf	START_D,f
	movf	ACC2_D+1,w
	addwfc	START_D+1,f
	movf	ACC2_D+2,w
	addwfc	START_D+2,f
	movf	ACC2_D+3,w
	addwfc	START_D+3,f
; check minimum
	btfsc	START_D+3,7	;check overflow
	goto	FAS326
	movf	START_D+3,w	;min=10KHz
	BNZ	FAS327
	movf	START_D+2,w
	BNZ	FAS327
	movlw	LOW d'10000'
	subwf	START_D,w
	movwf	ACC0_W
	movlw	HIGH d'10000'
	subwfb	START_D+1,w
	movwf	ACC0_W+1
	BC	FAS327
FAS326:	movlw	LOW d'10000'
	movwf	START_D
	movlw	HIGH d'10000'
	movwf	START_D+1
	clrf	START_D+2
	clrf	START_D+3
; check maximum
FAS327:	movff	START_D,ACC2_D
	movff	START_D+1,ACC2_D+1
	movff	START_D+2,ACC2_D+2
	movff	START_D+3,ACC2_D+3
	call	CALC_STOPF
	movlw	LOW d'15000000'	;(ACC2_D)-15MHz
	subwf	ACC2_D,f
	movlw	HIGH d'15000000'
	subwfb	ACC2_D+1,f
	movlw	UPPER d'15000000'
	subwfb	ACC2_D+2,f
	movlw	0
	subwfb	ACC2_D+3,f
	btfsc	ACC2_D+3,7	;check overflow
	goto	FAS1
;
	movlw	LOW d'15000000'	;(15MHz)
	movwf	START_D
	movlw	HIGH d'15000000'
	movwf	START_D+1
	movlw	UPPER d'15000000'
	movwf	START_D+2
	clrf	START_D+3
;
	movlw	d'128'		;128 step
	mulwf	STEP_W		;(ACC2_D)=128 * (STEP_W)
	movff	PRODL,ACC2_D
	movff	PRODH,ACC2_D+1
	clrf	ACC2_D+2
	clrf	ACC2_D+3
	movlw	d'128'
	mulwf	STEP_W+1
	movf	PRODL,w
	addwf	ACC2_D+1,f
	movf	PRODH,w
	addwfc	ACC2_D+2,f
	btfsc	STATUS,C
	incf	ACC2_D+3
;
	movf	ACC2_D,w
	subwf	START_D,f
	movf	ACC2_D+1,w
	subwfb	START_D+1,f
	movf	ACC2_D+2,w
	subwfb	START_D+2,f
	movf	ACC2_D+3,w
	subwfb	START_D+3,f
	goto	FAS1
;
; change step
FAS33:	btfsc	TMP0,7		;check sign
	goto	FAS335
	incf	STEPPTR,f
FAS332:	movlw	b'00001111'
	andwf	STEPPTR,f
	call	SET_STEP
	goto	FAS1
FAS335:	decf	STEPPTR,f
	goto	FAS332
; change dial mode
FAS35:	incf	FR_GA,f
	movlw	b'00000001'
	andwf	FR_GA,f
	goto	FAS1
; change display mode
FAS36:	incf	GA_PH,f
	movlw	b'00000011'
	andwf	GA_PH,f
	btfsc	STATUS,Z
	incf	GA_PH,f
	goto	FAS1
;
FAS_MODE_NEXT:
	clrf	MODEFY
	clrf	MODTYP
	clrf	OUTPUT
	clrf	WAVEFM
	clrf	FUNCTION
; control reg. - stop DDS
	movlw	b'00100001'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......1	;RESET
	movwf	ACC0_W+1
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	ACC0_W
	call	WRDDS
	movff	START_D,FREQ_D
	movff	START_D+1,FREQ_D+1
	movff	START_D+2,FREQ_D+2
	movff	START_D+3,FREQ_D+3
	goto	SG_MODE
; freq.Analysis Display mode ==============================================
FAD_MODE:
	call	LCD_CLR		;clear screen
	clrf	PX
	clrf	PY
	movlw	LOW M_MODE_SWEEP
	movwf	ACC0_W
	movlw	HIGH M_MODE_SWEEP
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	movlw	2
	movwf	FUNCTION
;
	clrf	ACC0_W		;stop TMR1 interrupt
	clrf	ACC0_W+1
	call	INIZ_TMR1
;
FAD_LOOP:
	movff	START_D,FREQ_D
	movff	START_D+1,FREQ_D+1
	movff	START_D+2,FREQ_D+2
	movff	START_D+3,FREQ_D+3
	clrf	PX
; control reg.
	movlw	b'00100001'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......1	;reset
	movwf	ACC0_W+1
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	ACC0_W
	call	WRDDS
; measure base offsetvoltage
	movlw	d'10'
	call	mSleep
	movlw	0		;high level detecotr
	call	ADC
	movff	ACC0_W,OFSETH
	movlw	1		;low level detector
	call	ADC
	movff	ACC0_W,OFSETL
; start DDS	
; control reg.
	movlw	b'00100000'	;DDS control register
;		  00......	;control register select
;		  ..10....	;B28 select - freq.data is 28 bit
;		  ....0...	;FREQ0 register select
;		  .....0..	;PHASE0 register select
;		  ......0.	;PIN/SW - software conteol mode
;		  .......0	;reset release
	movwf	ACC0_W+1
	movlw	b'00000000'
;		  00......	;not sleep
;		  ..000...	;no sign bit out
;		  .....0.0	;always zero
;		  ......0.	;sign wave mode
	movwf	ACC0_W
	call	WRDDS
FADL1:	
; calc. and set frequency
	movff	FREQ_D,ACC2_D	;calc. frequency
	movff	FREQ_D+1,ACC2_D+1
	movff	FREQ_D+2,ACC2_D+2
	movff	FREQ_D+3,ACC2_D+3
	call	CALC_FREQ

	movff	ACC2_D,ACC0_W	;set to DDS FREQ0 register
	movff	ACC2_D+1,ACC0_W+1
	bsf	ACC0_W+1,6
	call	WRDDS
	movff	ACC2_D+2,ACC0_W
	movff	ACC2_D+3,ACC0_W+1
	bsf	ACC0_W+1,6
	call	WRDDS

	movff	ACC2_D,ACC0_W	;set to DDS FREQ1 register
	movff	ACC2_D+1,ACC0_W+1
	bsf	ACC0_W+1,7
	call	WRDDS
	movff	ACC2_D+2,ACC0_W
	movff	ACC2_D+3,ACC0_W+1
	bsf	ACC0_W+1,7
	call	WRDDS
;
	movlw	d'10'		;acquisition time
	call	mSleep
; measure and display
	btfss	GA_PH,0		;gain display enable?
	goto	FADL16
; gain display
	movlw	0		;get high level detector
	call	ADC
	movf	OFSETH,w
	subwf	ACC0_W,f
	btfss	STATUS,C
	decf	ACC0_W+1,f
	btfss	ACC0_W+1,7	;check sign(overflow)
	goto	FADL11
	clrf	ACC0_W
	clrf	ACC0_W+1
FADL11:	movf	ACC0_W+1,w
	BNZ	FADL12
	movlw	d'50'
	subwf	ACC0_W,w
	BNC	FADL13
;
FADL12:	bcf	STATUS,C
	rlcf	ACC0_W,f		;*2
	rlcf	ACC0_W+1,f
	rlcf	ACC0_W,f		;*4
	rlcf	ACC0_W+1,f
	rlcf	ACC0_W,f		;*8
	rlcf	ACC0_W+1,f
	rlcf	ACC0_W,f		;*16
	rlcf	ACC0_W+1,f
	movff	ACC0_W,ACC1_W	;copy it
	movff	ACC0_W+1,ACC1_W+1
	goto	FADL15

FADL13:	movlw	1		;get low level detector
	call	ADC
	movf	OFSETL,w
	subwf	ACC0_W,f
	btfss	STATUS,C
	decf	ACC0_W+1,f
	btfss	ACC0_W+1,7	;check sign(overflow)
	goto	FADL14
	clrf	ACC0_W
	clrf	ACC0_W+1
FADL14:	movff	ACC0_W,ACC1_W
	movff	ACC0_W+1,ACC1_W+1	;copy data
; currentry, gain data into ACC1_W(lower word)
FADL15:	movff	GAIN_W,ACC0_W	;gain adjust
	movff	GAIN_W+1,ACC0_W+1
	call	MUL1616
	call	DIV10
	movlw	d'63'
	movwf	ACC0_W		;invert it
	movf	ACC2_D+1,w
	subwf	ACC0_W,f
	clrf	ACC0_W+1
;
	movff	PX,TMP0		;put a dot to graphics scr.
	call	LCD_CLR_VLINE
	movff	PX,TMP0
	movff	ACC0_W,TMP1
	call	LCD_PSET
; phase display
FADL16:	btfss	GA_PH,1
	goto	FADL19
	movlw	2		;get phase detector
	call	ADC
	rrcf	ACC0_W+1,f	;adjust pos.
	rrcf	ACC0_W,f
	rrcf	ACC0_W+1,f
	rrcf	ACC0_W,f
	rrcf	ACC0_W+1,f
	rrcf	ACC0_W,f
	rrcf	ACC0_W+1,f
	rrcf	ACC0_W,f
	movff	PX,TMP0		;put a dot to graphics scr.
	call	LCD_CLR_VLINE
	movff	PX,TMP0
	movff	ACC0_W,TMP1
	call	LCD_PSET
;
FADL19:	movf	STEP_W,w
	addwf	FREQ_D,f
	movf	STEP_W+1,w
	addwfc	FREQ_D+1,f
	movlw	0
	addwfc	FREQ_D+2,f
	movlw	0
	addwfc	FREQ_D+3,f

	incf	PX,f
	movlw	d'96'
	subwf	PX,w
	BZ	FADL2
	goto	FADL1
; check switch and rotary encoder
FADL2:	movff	PORTC,TMP0
	movlw	b'11000111'
	iorwf	TMP0,f
	incf	TMP0,w
	BZ	FADL3		;not press
	btfss	TMP0,PSW3
	goto	FASET_MODE
; check rotary encoder
FADL3:	movf	RECOUNT,w
	movwf	TMP0
	BNZ	FADL4
	goto	FAD_LOOP
FADL4:	movf	FR_GA,w
	BNZ	FADL5
; change start freq.
	clrf	RECOUNT
	movff	TMP0,ACC0_W
	clrf	ACC0_W+1
	comf	TMP0,w
	addlw	1
	btfsc	ACC0_W,7		;check sign
	movwf	ACC0_W
	movff	STEP_W,ACC1_W
	movff	STEP_W+1,ACC1_W+1
	call	MUL1616
	btfss	TMP0,7		;check sign
	goto	FADL41
	comf	ACC2_D,f
	comf	ACC2_D+1,f
	comf	ACC2_D+2,f
	comf	ACC2_D+3,f
	incf	ACC2_D,f
	btfsc	STATUS,Z
	incf	ACC2_D+1,f
	btfsc	STATUS,Z
	incf	ACC2_D+2,f
	btfsc	STATUS,Z
	incf	ACC2_D+3,f
FADL41:	movf	ACC2_D,w
	addwf	START_D,f
	movf	ACC2_D+1,w
	addwfc	START_D+1,f
	movf	ACC2_D+2,w
	addwfc	START_D+2,f
	movf	ACC2_D+3,w
	addwfc	START_D+3,f
; check minimum
	btfsc	START_D+3,7	;check overflow
	goto	FADL42
	movf	START_D+3,w	;min=10KHz
	BNZ	FADL43
	movf	START_D+2,w
	BNZ	FADL43
	movlw	LOW d'10000'
	subwf	START_D,w
	movwf	ACC0_W
	movlw	HIGH d'10000'
	subwfb	START_D+1,w
	movwf	ACC0_W+1
	BC	FADL43
FADL42:	movlw	LOW d'10000'
	movwf	START_D
	movlw	HIGH h'10000'
	movwf	START_D+1
	clrf	START_D+2
	clrf	START_D+3
; check maximum
FADL43:	movlw	LOW d'15000000'	;(15MHz) - (FREQ_D)
	movwf	ACC2_D
	movlw	HIGH d'15000000'
	movwf	ACC2_D+1
	movlw	UPPER d'15000000'
	movwf	ACC2_D+2
	clrf	ACC2_D+3
	movf	START_D,w
	subwf	ACC2_D,f
	movf	START_D+1,w
	subwfb	ACC2_D+1,f
	movf	START_D+2,w
	subwfb	ACC2_D+2,f
	movf	START_D+3,w
	subwfb	ACC2_D+3,f
	btfss	ACC2_D+3,7	;check overflow
	goto	FAD_LOOP
;
	movlw	LOW d'15000000'	;(15MHz)
	movwf	START_D
	movlw	HIGH d'15000000'
	movwf	START_D+1
	movlw	UPPER d'15000000'
	movwf	START_D+2
	clrf	START_D+3
	goto	FAD_LOOP
; change gain
FADL5:	movff	RECOUNT,ACC0_W
	clrf	RECOUNT
	clrf	ACC0_W+1
	btfsc	ACC0_W,7		;sign extend
	comf	ACC0_W+1,f
;
	movf	ACC0_W,w		;add to GAIN_W
	addwf	GAIN_W,f
	movf	ACC0_W+1,w
	addwfc	GAIN_W+1,f
	btfsc	ACC0_W+1,7	;check sign
	goto	FADL52
	movlw	b'11111100'
	andwf	GAIN_W+1,w
	BNZ	FADL51
	goto	FAD_LOOP
FADL51:	movlw	h'03'
	movwf	GAIN_W+1
	movlw	h'ff'
	movwf	GAIN_W	
	goto	FAD_LOOP
FADL52:	btfss	GAIN_W+1,7
	goto	FAD_LOOP
	movlw	d'1'
	movwf	GAIN_W
	clrf	GAIN_W+1
	goto	FAD_LOOP
	

; INTERRUPT HANDLER - high priority - TMR0(500Hz) =========================
INTERRUPT_HIGH:
	btfsc	INTCON,TMR0IF
	goto	IHTMR0
	retfie
IHTMR0:
	movwf	W_SAVE		;context save
	movff	STATUS,S_SAVE
	bcf	INTCON,TMR0IF	;clear interrupt flag
	movlw	HIGH -d'500'
	movwf	TMR0H
	movlw	LOW -d'500'
	movwf	TMR0L
; check rotry encoder
	movf	PORTC,w		;check for Rotary Encoder phase A bit
	xorwf	RESAVE,w
	andlw	REABIT
	BZ	IHT9		;no change
	movff	PORTC,RESAVE
	btfsc	RESAVE,REA	;fall edge ?
	goto	IHT9
	btfss	RESAVE,REB	;direction ?
	goto	IHT2
	movlw	d'1'		;slow or fast
	btfss	PORTB,REP
	movlw	d'10'
	addwf	RECOUNT,f	;count up
	goto	IHT9
IHT2:	movlw	d'1'		;slow or fast
	btfss	PORTB,REP
	movlw	d'10'
	subwf	RECOUNT,f
;	goto	IHT9
IHT9:	movf	W_SAVE,w		;restore context
	movff	S_SAVE,STATUS
	retfie

INTERRUPT_LOW:
	btfsc	PIR1,RCIF
	goto	IRCHR
	btfsc	PIR1,TMR1IF
	goto	ITMR1
	retfie
IRCHR:	movwf	W_SAVE
	movff	STATUS,S_SAVE
	movff	FSR0L,F_SAVE
	movff	FSR0H,F_SAVE+1
;	bcf	PIR1,RCIF
	movf	RXCNT,w		;check receive buffer over
	iorlw	b'11000000'
	xorwf	b'11111111'
	BZ	IRCHR9
	movf	RXWPTR,w		;calc.write addr.
	addlw	LOW RXBUF
	movwf	FSR0L
	movlw	HIGH RXBUF
	movwf	FSR0H
	movf	RCREG,w		;copy receipt chr.
	movwf	INDF0
	movwf	LASTCHR
	incf	RXCNT,f		;counter and pointer update
	incf	RXWPTR,f
	movlw	b'00111111'
	andwf	RXWPTR,f
IRCHR9:	movf	RCREG,w		;force clear RCIF
ILEND:	movff	FSR0L,F_SAVE
	movff	FSR0H,F_SAVE+1
	movf	W_SAVE,w
	movff	S_SAVE,STATUS
	retfie
;
ITMR1:	movwf	W_SAVE
	movff	STATUS,S_SAVE
	bcf	PIR1,TMR1IF
	bcf	T1CON,TMR1ON	;stop TMR1
	movff	T1TMG_W,TMR1L	;set new value
	movff	T1TMG_W+1,TMR1H
	bsf	T1FLAG,0		;set flag
	bsf	T1CON,TMR1ON	;restart
	movf	W_SAVE,w
	movff	S_SAVE,STATUS
	retfie

; subroutine INIZ_PORT - PORTA..E initialize
;	input/output	nohting
INIZ_PORT:
	clrf	PORTA
	clrf	LATA
	clrf	PORTB
	clrf	LATB
	movlw	b'00000100'
	movwf	PORTC
	movwf	LATC
	clrf	PORTD
	clrf	LATD
	movlw	b'00000111'
	movwf	PORTE
	movwf	LATE

	movlw	b'00111011'	;ADCON1
;		  ..00....	;refernce is Vss/Vss
;		  ....1011	;AD input is RA0..RA3
	movwf	ADCON1

	movlw	b'00001111'	;PORTA
;		  ..00....	;not use
;		  ....1...	;battery voltage(analog)
;		  .....1..	;phase sense input(analog)
;		  ......1.	;signal level high gain(analog)
;		  .......1	;signal level low gain(analog)
	movwf	TRISA
	movlw	b'00100000'	;PORTB
;		  00......	;ICSP
;		  ..1.....	;rotary encoder push switch
;		  ...0....	;LCD RS
;		  ....0...	;LCD R/W
;		  .....0..	;LCD E
;		  ......0.	;LCD CS1
;		  .......0	;LCD CS2
	movwf	TRISB
	movlw	b'10111011'	;PORTC
;		  10......	;EUSART
;		  ..000...	;push switch SW1..3
;		  .....0..	;DDS RESET
;		  ......00	;rotary encoder phase A and B
	movwf	TRISC
	movlw	b'11111111'	;PORTD - LCD data(currentry hi-Z)
	movwf	TRISD
	movlw	b'00000000'	;PORTE - DDS
;		  .....0..	;DDS serial clock
;		  ......0.	;DDS serial data
;		  .......0	;DDS flame sync
	movwf	TRISE
	return

; subroutine INIZ_TMR0 - for 500Hz interrupt
;	input/output	nothing
INIZ_TMR0:
	bcf	INTCON,TMR0IE	
	clrf	TMR0H
	clrf	TMR0L
	movlw	b'10010011'
;		  1.......	;TMR0 ON
;		  .0......	;16bit mode
;		  ..0.....	;input is instruction cycle
;		  ...0....	;rize edge
;		  ....0...	;pre-scaler assign
;		  .....011	;1/16
	movwf	T0CON
	bcf	INTCON,TMR0IF
	bsf	RCON,IPEN	;interrput priority enable
	bsf	INTCON2,TMR0IP	;high priority
	bsf	INTCON,TMR0IE
	movlw	HIGH -d'500'
	movwf	TMR0H
	movlw	LOW -d'500'
	movwf	TMR0L
	bsf	INTCON,TMR0IE
	bsf	INTCON,GIE
	return

; subroutine INIZ_TMR1 - for frequency counter with TMR0 interrupt
;	input	ACC0_W=freq.(Hz) if zero, stop TMR1 interrupt
;	output	nothing
INIZ_TMR1:
	movf	ACC0_W,w		;check if zero
	iorwf	ACC0_W+1,w
	BNZ	IZT12
IZT10:	bcf	T1CON,TMR1ON	;stop TMR1
	bcf	PIE1,TMR1IE	;stop interrupt
	bcf	PIR1,TMR1IF
	return
; calc. T1 timing data
IZT12:
;	bcf	STATUS,C		;*2
;	rlcf	ACC0_W,f
;	rlcf	ACC0_W+1,f
	movlw	LOW d'2000000'
	movwf	ACC2_D
	movlw	HIGH d'2000000'
	movwf	ACC2_D+1
	movlw	UPPER d'2000000'
	movwf	ACC2_D+2
	clrf	ACC2_D+3
	call	DIV3216		;
	BC	IZT10
	clrf	T1TMG_W
	clrf	T1TMG_W+1
	movf	ACC2_D,w
	subwf	T1TMG_W,f
	movf	ACC2_D+1,w
	subwfb	T1TMG_W+1,f
	movlw	d'2'		;timing adjust
	addwf	T1TMG_W,f
	btfsc	STATUS,C
	incf	T1TMG_W+1,f
;
IZT13:	clrf	T1FLAG
	bcf	IPR1,TMR1IP
	bcf	PIR1,TMR1IF
	movff	T1TMG_W,TMR1L
	movff	T1TMG_W+1,TMR1H
	movlw	b'00010101'
;		  0.......	;read 8bit mode
;		  .0......	;source is not T1OSC
;		  ..01....	;prescaler 1:2
;		  ....0...	;T1OSC disable
;		  .....1..	;no syncronize
;		  ......0.	;external input(T1CKI)
;		  .......1	;TMR1 ON
	movwf	T1CON
	bsf	PIE1,TMR1IE	;interrupt enable
	return

; subroutine INIZ_COM - EUSART initialize
;	9600bps, 8bit, no parity
;	no receive interrupt
INIZ_COM:
; SPBRG
	movlw	HIGH d'417'	;for 19200bps
	movwf	SPBRGH
	movlw	LOW d'417'
	movwf	SPBRG
; BAUDCON
	movlw	b'00001000'
;		  .0......	;receiver active
;		  ....1...	;BRG 16 bit mode
;		  ......0.	;no wakeup
;		  .......0	;auto baud rate disable
	movwf	BAUDCON
; RCSTA
	movlw	b'10010000'
;		  1.......	;Serial Port enable
;		  .0......	;RX 8 bit mode
;		  ...1....	;continuous receive enable
;		  .....*..	;flaming error
;		  ......*.	;overrun error
	movwf	RCSTA
; TXSTA
	movlw	b'00100100'
;		  .0......	;TX 8 bit mode
;		  ..1.....	;transmitter enable
;		  ...0....	;Async mode
;		  ....0...	;normal operation
;		  .....1..	;BRGH high speed
;		  ......*.	;transmitter empty
	movwf	TXSTA
;
	clrf	RXRPTR
	clrf	RXWPTR
	clrf	RXCNT
	clrf	LASTCHR
;
	bcf	IPR1,RCIP	;RC interrupt low priority
	bcf	PIR1,RCIF	;RC interrupt enable
	bsf	PIE1,RCIE
	bsf	INTCON,GIEL

 	return


; subroutine INIZ_LCD
;	input	nothing
;	output	nothing
INIZ_LCD:
	movlw	d'100'		;wait for powerup
	call	mSleep
	call	LCD_STAT
	bcf	PORTB,LRS	;command register
	bsf	PORTB,LCSR	;left side
	bcf	PORTB,LCSL
	movlw	b'00111111'	;display ON
	call	LCD_WR
	call	LCD_STAT
	movlw	b'11000000'	;start line is 0
	call	LCD_WR
	call	LCD_STAT
	bcf	PORTB,LCSR	;right side
	bsf	PORTB,LCSL
	movlw	b'00111111'	;display ON
	call	LCD_WR
	call	LCD_STAT
	call	LCD_STAT
	movlw	b'11000000'	;start line is 0
	call	LCD_WR

	call	LCD_CLR
	return


; subroutine LCD_STAT
;	input	nothing
;	output	nothing
;	affeect	TMP2
LCD_STAT:
	bcf	INTCON,GIE
	movff	PORTB,TMP2	;save chip select
	bcf	PORTB,LRS	;status read
	bsf	PORTB,LCSR	;select left driver
	bcf	PORTB,LCSL
LST1:	clrwdt
	call	LCD_RD
	andlw	b'10000000'
	BNZ	LST1
	bcf	PORTB,LCSR	;select right driver
	bsf	PORTB,LCSL
LST2:	clrwdt
	call	LCD_RD
	andlw	b'10000000'
	BNZ	LST2
	movff	TMP2,PORTB	;restore chip select
	bsf	INTCON,GIE
	return

; subroutine LCD_WR - write to LCD
;	input	W=data
;	output	nothing
LCD_WR:	bcf	INTCON,GIE
	movwf	PORTD		;set data and output
	clrf	TRISD
	bcf	PORTB,LRW	;WR mode
	WENB			;make setup time
	bsf	PORTB,LE		;start E signal
	WENB	
	bcf	PORTB,LE		;end of E signal
	nop			;make hold time
	nop
	bsf	PORTB,LRW
	movlw	b'11111111'	;bus output disable
	movwf	TRISD
	bsf	INTCON,GIE
	return

; subroutine LCD_RD - read from LCD
;	input	nothing
;	output	W = data
LCD_RD:
	bcf	INTCON,GIE
	movlw	b'11111111'	;bus input mode
	movwf	TRISD
	bsf	PORTB,LRW	;RD mode
	WENB			;make setup time
	bsf	PORTB,LE		;start E signal
	WENB
	movf	PORTD,w		;read data
	bcf	PORTB,LE		;end of E signal
	nop			;make hold time
	bsf	INTCON,GIE
	return

; subroutine LCD_RD2 - read from LCD with twice Enable
;	input	nothing
;	output	W = data
LCD_RD2:
	bcf	INTCON,GIE
	movlw	b'11111111'	;bus input mode
	movwf	TRISD
	bsf	PORTB,LRW	;RD mode
	WENB			;make setup time
	bsf	PORTB,LE		;start E signal
	WENB
	bcf	PORTB,LE		;end of E signal
	WENB
	bsf	PORTB,LE		;start E signal
	WENB
	movf	PORTD,w		;read data
	bcf	PORTB,LE		;end of E signal
	nop			;make hold time
	bsf	INTCON,GIE
	return

; subroutine LCD_CLR
;	input	nothing
;	output	nothing
;	affect	TMP0,TMP1
LCD_CLR:
	bsf	PORTB,LCSR
	bcf	PORTB,LCSL
	call	LCLR5
	bcf	PORTB,LCSR
	bsf	PORTB,LCSL
LCLR5:	clrf	TMP0		;Y pos.
LCLR6:	call	LCD_STAT
	movlw	b'10111000'	;set page commane
	iorwf	TMP0,w
	bcf	PORTB,LRS	;command register
	call	LCD_WR
;	
	clrf	TMP1		;X pos.
LCLR7:	call	LCD_STAT
	movlw	b'01000000'	;set address command
	iorwf	TMP1,w
	bcf	PORTB,LRS	;command register
	call	LCD_WR
	call	LCD_STAT
	bsf	PORTB,LRS	;data register
	movlw	b'00000000'
	call	LCD_WR
	incf	TMP1,f
	btfss	TMP1,6		;64?
	goto	LCLR7
	incf	TMP0,f
	btfss	TMP0,3		;8?
	goto	LCLR6
	return

; subroutine LCD_PSET
;	input	TMP0 = X position
;		TMP1 = Y position
;	output	nothing
;	affect	TMP0,TMP1,TMP2
LCD_PSET:
	btfsc	TMP0,6		;left or right
	goto	lps1
	bcf	PORTB,LCSL	;left side select
	bsf	PORTB,LCSR
	goto	lps2
lps1:	bsf	PORTB,LCSL	;right side select
	bcf	PORTB,LCSR
lps2:	call	LCD_STAT
	movlw	b'00111111'
	andwf	TMP0,w
	iorlw	b'01000000'	;set X position
	bcf	PORTB,LRS
	call	LCD_WR
;
	movff	TMP1,TMP0
	rrcf	TMP0,f
	rrcf	TMP0,f
	rrcf	TMP0,f
	call	LCD_STAT
	movlw	b'00000111'
	andwf	TMP0,w
	iorlw	b'10111000'	;set Y position(byte)
	bcf	PORTB,LRS
	call	LCD_WR
;
	movlw	b'00000001'	;make bit pattern
	movwf	TMP0
	movlw	b'00000111'
	andwf	TMP1,f
lps3:	movf	TMP1,w
	BZ	lps4
	decf	TMP1,f
	bcf	STATUS,C
	rlcf	TMP0,f
	goto	lps3
lps4:	call	LCD_STAT
	movf	TMP0,w
	bsf	PORTB,LRS
	call	LCD_WR		;write
	return

; subroutine LCD_CLR_VLINE
;	input	TMP0 = X position
;	output	nothing
;	affect	TMP0,TMP1,TMP2
LCD_CLR_VLINE:
	btfsc	TMP0,6		;left or right
	goto	lcv1
	bcf	PORTB,LCSL	;left side select
	bsf	PORTB,LCSR
	goto	lcv2
lcv1:	bsf	PORTB,LCSL	;right side select
	bcf	PORTB,LCSR
lcv2:	call	LCD_STAT
	movlw	b'00111111'
	andwf	TMP0,w
	iorlw	b'01000000'	;set X position
	bcf	PORTB,LRS
	call	LCD_WR
;
	clrf	TMP1
lcv3:	call	LCD_STAT
	bcf	PORTB,LRS
	movlw	b'10111000'	;set Y position(top)
	iorwf	TMP1,w
	call	LCD_WR
	call	LCD_STAT
	bsf	PORTB,LRS
	movlw	0
	call	LCD_WR		;write
	incf	TMP1,f
	btfss	TMP1,3
	goto	lcv3

	return

; subroutine PUTC_LCD - put a character
;	input	W=data, (PX, PY) = position
;	output	PX += 6, PY update
;	affect	TMP0,TMP1,TMP2,PX,PY
PUTC_LCD:
; clac.font offset
	movwf	TMP0
	bcf	TMP0,7		;mask for 0..0x7f
	movlw	h'20'
	subwf	TMP0,f
	btfss	STATUS,C
	return			;when W < 0x20, do nothing
	clrf	TMP1
	bcf	STATUS,C
	rlcf	TMP0,f		;*2
	movf	TMP0,w
	bcf	STATUS,C
	rlcf	TMP0,f		;*4
	rlcf	TMP1,f
	addwf	TMP0,f		;*6
	btfsc	STATUS,C
	incf	TMP1,f
; set font address
	movlw	LOW FONT5x7
	addwf	TMP0,f
	movlw	HIGH FONT5x7
	btfsc	STATUS,C
	movlw	(HIGH FONT5x7)+1
	addwf	TMP1,f
	movff	TMP0,TBLPTRL
	movff	TMP1,TBLPTRH
	clrf	TBLPTRU
; set LCD Y position
	movf	PY,w
	andlw	b'00000111'
	iorlw	b'10111000'	;LCD set page command
	movwf	TMP0
	bcf	PORTB,LRS	;RS=command reg.
	bsf	PORTB,LCSR	;left side select
	bcf	PORTB,LCSL
	call	LCD_STAT
	bcf	PORTB,LRS
	movf	TMP0,w
	call	LCD_WR
	bcf	PORTB,LCSR	;right side select
	bsf	PORTB,LCSL
	call	LCD_STAT
	movf	TMP0,w
	call	LCD_WR
; set LCD X position
	bcf	PX,7		;mask X pos
	btfsc	PX,6
	goto	pcl2
pcl1:	bcf	PORTB,LCSL	;lsft side
	bsf	PORTB,LCSR
	goto	pcl3
pcl2:	bcf	PORTB,LCSR	;right side
	bsf	PORTB,LCSL
pcl3:	call	LCD_STAT
	bcf	PORTB,LRS
	movf	PX,w
	andlw	b'00111111'
	iorlw	b'01000000'
	call	LCD_WR
; transfer FLASH to LCD
	movlw	d'6'		;X dot count
	movwf	TMP0
pcl5:	call	LCD_STAT
	bsf	PORTB,LRS	;RS=data reg.
	TBLRD*+
	movf	TABLAT,w
	call	LCD_WR
	incf	PX,f
	movlw	d'64'		;check boundary
	subwf	PX,w
	BNZ	pcl6
	bsf	PORTB,LCSL	;right side
	bcf	PORTB,LCSR
	bcf	PORTB,LRS
	movlw	b'01000000'	;address reset
	call	LCD_WR
	bsf	PORTB,LRS
pcl6:	movlw	d'126'		;check end of line
	subwf	PX,w
	BNZ	pcl7
	incf	PY,f		;next line(PY not check)
	clrf	PX
	bcf	PORTB,LCSL	;left side
	bsf	PORTB,LCSR
pcl7:	decfsz	TMP0,f
	goto	pcl5
	return

; subroutine PSTRING_LCD (SBUF/ASCIZ)
;	input	(SBUF)
;	output	nothing
;	affect	TMP0,TMP1,FSR0,PX
PSTRING_LCD:
	movlw	LOW SBUF
	movwf	FSR0L
	clrf	FSR0H
PSL1:	movf	INDF0,w
	btfsc	STATUS,Z
	return
	call	PUTC_LCD
	incf	FSR0L,f
	goto	PSL1

; subroutine PSTRING_LCD_FLASH (SBUF/ASCIZ)
;	input	ACC0_W=flash memory address
;	output	nothing
;	affect	TMP0,TMP1,ACC0_W,PX
PSTRING_LCD_FLASH:
	movff	ACC0_W,TBLPTRL
	movff	ACC0_W+1,TBLPTRH
	clrf	TBLPTRU
	TBLRD*+
	movf	TABLAT,w
	btfsc	STATUS,Z
	return
	call	PUTC_LCD
	incf	ACC0_W,f
	btfsc	STATUS,Z
	incf	ACC0_W+1,f
	goto	PSTRING_LCD_FLASH

; subroutine mSleep - sleep mS
;	inout	W=time(mS)
;	output	nothing
;	affect	TMP0,TMP1
mSleep:
	movwf	TMP1
mS1:	movlw	d'190'
	movwf	TMP0
mS2:	goto	$+2
	goto	$+2
	goto	$+2
	goto	$+2
	goto	$+2
	goto	$+2
	decfsz	TMP0,f
	goto	mS2
	clrwdt
	decfsz	TMP1,f
	goto	mS1
	return

; subroutine PUTC_COM
;	input	W=data
;	output	nothing
PUTC_COM:
	clrwdt
	btfss	TXSTA,TRMT
	goto	PUTC_COM
	movwf	TXREG
	return

; subroutine PCRLF - put CR/LF
PCRLF:
	movlw	h'0D'
	call	PUTC_COM
	movlw	h'0A'
	call	PUTC_COM
	return

; subroutine PUTHEX_COM
;	input	W=data
;	output	nothing
;	affect	TMP2
PUTHEX_COM:
	movwf	TMP2
	swapf	TMP2,w
	call	PHX1
	movf	TMP2,w
PHX1:	andlw	b'00001111'
	sublw	d'9'
	btfsc	STATUS,C
	goto	PHX2
	sublw	d'16'
	goto	PHX3
PHX2:	sublw	d'9'
PHX3:	addlw	'0'
	call	PUTC_COM
	return
	

; subroutine PSTRING - put string(SBUF/ASCIZ)
;	input	nothing
;	output	nothing
;	affect	FSR0
PSTRING:
	movlw	LOW SBUF
	movwf	FSR0L
	movlw	HIGH SBUF
	movwf	FSR0H
PS1:	movf	INDF0,w
	btfsc	STATUS,Z
 return
;	goto	PCRLF
	call	PUTC_COM
	incf	FSR0L,f
	goto	PS1

; subroutine MUL1616
;	input	ACC0_W,ACC1_W
;	output	ACC2_D
MUL1616:
	movf	ACC0_W,w		;L * L
	mulwf	ACC1_W
	movff	PRODH,ACC2_D+1
	movff	PRODL,ACC2_D

	movf	ACC0_W+1,w	;H * H
	mulwf	ACC1_W+1
	movff	PRODH,ACC2_D+3
	movff	PRODL,ACC2_D+2

	movf	ACC0_W,w		;L * H
	mulwf	ACC1_W+1
	movf	PRODL,w
	addwf	ACC2_D+1,f
	movf	PRODH,w
	addwfc	ACC2_D+2,f
	clrf	WREG
	addwfc	ACC2_D+3,f

	movf	ACC0_W+1,w	;H * L
	mulwf	ACC1_W
	movf	PRODL,w
	addwf	ACC2_D+1,f
	movf	PRODH,w
	addwfc	ACC2_D+2,f
	clrf	WREG
	addwfc	ACC2_D+3,f
	return

; subroutine DIV3216 - divide 32 bit/ 16 bit(unsigned)
;	input	ACC2_D, ACC0_W
;	output	(ACC2_D)=(ACC2_D)/(ACC0_W)
;		(ACC1_W)=modurous
;		when Carry, error.
;	affect	TMP0,TMP1,TMP2,ACC0_W,ACC1_W,ACC2_D
DIV3216:
	movf	ACC0_W,w
	iorwf	ACC0_W+1,w
	BZ	div9
;
	clrf	ACC1_W		;clear modurous
	clrf	ACC1_W+1
	movlw	d'32'
	movwf	TMP0		;bit counter
div1:	bcf	STATUS,C
	rlcf	ACC2_D,f
	rlcf	ACC2_D+1,f
	rlcf	ACC2_D+2,f
	rlcf	ACC2_D+3,f
	rlcf	ACC1_W,f
	rlcf	ACC1_W+1,f
	movf	ACC0_W,w
	subwf	ACC1_W,w
	movwf	TMP1		;save it
	movf	ACC0_W+1,w
	subwfb	ACC1_W+1,w
	movwf	TMP2		;(TMP2:TMP1)=(ACC1_W)-(ACC0_W)
	btfsc	TMP2,7
	goto	div3
div2:	bsf	ACC2_D,0
	movf	TMP2,w		;copy
	movwf	ACC1_W+1
	movf	TMP1,w
	movwf	ACC1_W
div3:	decfsz	TMP0,f
	goto	div1
	bcf	STATUS,C		;no error
	return
div9:	bsf	STATUS,C		;error
	return

; subroutine DIV10 - ACC2_D divide by 10
;	input	ACC2_D
;	output	ACC2_D, TMP0=modurous
;	affect	ACC2_D,TMP0,TMP1
DIV10:
	movlw	d'32'	;bit counter
	movwf	TMP1
	clrf	TMP0	;modurous
DV101:	bcf	STATUS,C
	rlcf	ACC2_D,f
	rlcf	ACC2_D+1,f
	rlcf	ACC2_D+2,f
	rlcf	ACC2_D+3,f
	rlcf	TMP0,f
	movlw	d'10'
	subwf	TMP0,w
	BNC	DV102
	bsf	ACC2_D,0
	movwf	TMP0
DV102:	decfsz	TMP1,f
	goto	DV101
	return

; subroutine PUTDEC - make decimal string(10 chr+\0)
;	input	ACC2_D
;	output	(SBUF)
;	affect	ACC2_D,TMP0,TMP1,TMP2,FSR0
PUTDEC:
	movlw	d'10'
	movwf	TMP2
	movlw	LOW (SBUF+9)
	movwf	FSR0L
	movlw	HIGH (SBUF+9)
	movwf	FSR0H
PD1:	call	DIV10
	movlw	'0'
	addwf	TMP0,w
	movwf	INDF0
	decf	FSR0L,f
	decfsz	TMP2,f
	goto	PD1
	clrf	SBUF+d'10'
	return
	
; subroutine WRDDS - write to DDS chip
;	inout	ACC0_W = data(16bit)
;	output	nothing
;	affect	TMP0,ACC0_W
WRDDS:
	movlw	d'16'
	movwf	TMP0
	bcf	PORTE,DSY	;FSYNC = L
WDD1:	rlcf	ACC0_W,f
	rlcf	ACC0_W+1,f
	btfsc	STATUS,C
	goto	WDD2
	bcf	PORTE,DDA	;SDATA
	goto	WDD3
WDD2:	bsf	PORTE,DDA
WDD3:	nop
	bcf	PORTE,DCK	;SCLK = L
	nop
	bsf	PORTE,DCK	;SCLK = H
	nop
	decfsz	TMP0,f
	goto	WDD1
	bsf	PORTE,DSY
	return

; subroutine CALC_FREQ
;	input	ACC2_D = frequency(Hz), max 20MHz
;	output	ACC2_D = DDS set data(2x14bit data)
;	affect	A48,ACC2_D,TMP0,TMP1,TMP2,ACC0_W,ACC1_W
CALC_FREQ:
	clrf	A48	;clear lower 24 bit
	clrf	A48+1
	clrf	A48+2
	movff	ACC2_D,A48+3
	movff	ACC2_D+1,A48+4
	movff	ACC2_D+2,A48+5
	movff	ACC2_D+3,TMP0
	movlw	b'00000001'	;mask to max freq.
	andwf	TMP0,f		;(33M554K432Hz)
	rrcf	TMP0,f		;/2
	rrcf	A48+5,f
	rrcf	A48+4,f
	rrcf	A48+3,f
	rrcf	A48+2,f
	rrcf	TMP0,f		;/4
	rrcf	A48+5,f
	rrcf	A48+4,f
	rrcf	A48+3,f
	rrcf	A48+2,f
	rrcf	TMP0,f		;/8
	rrcf	A48+5,f
	rrcf	A48+4,f
	rrcf	A48+3,f
	rrcf	A48+2,f
; A48=(FREQ_W)*2^21
	movlw	h'E1'		;(ACC2_D)=50MHz/128
	movwf	ACC2_D
	movlw	h'F5'
	movwf	ACC2_D+1
	movlw	h'05'
	movwf	ACC2_D+2
	clrf	ACC2_D+3
; A48/ACC2_D
	clrf	ACC1_W		;modurous(24bit)
	clrf	ACC1_W+1
	clrf	TMP1
	movlw	d'48'		;bit counter
	movwf	TMP0
CF01:	bcf	STATUS,C		;bit shift
	rlcf	A48,f
	rlcf	A48+1,f
	rlcf	A48+2,f
	rlcf	A48+3,f
	rlcf	A48+4,f
	rlcf	A48+5,f
	rlcf	ACC1_W,f
	rlcf	ACC1_W+1,f
	rlcf	TMP1,f
	movf	ACC2_D,w		;(TMP2:ACC0_W)=(TMP1:ACC1_W)-(ACC2_D)
	subwf	ACC1_W,w
	movwf	ACC0_W
	movf	ACC2_D+1,w
	btfss	STATUS,C
	incf	ACC2_D+1,w
	subwf	ACC1_W+1,w
	movwf	ACC0_W+1
	movf	ACC2_D+2,w
	btfss	STATUS,C
	incf	ACC2_D+2,w
	subwf	TMP1,w
	movwf	TMP2
	btfss	STATUS,C
	goto	CF02
	bsf	A48,0		;set LSB
	movff	ACC0_W,ACC1_W
	movff	ACC0_W+1,ACC1_W+1
	movff	TMP2,TMP1
CF02:	decfsz	TMP0,f
	goto	CF01
; move to 14bit x2
	movff	A48,ACC2_D
	movff	A48+1,ACC2_D+1
	bcf	STATUS,C
	rlcf	A48+1,f
	rlcf	A48+2,f
	rlcf	A48+3,f
	bcf	STATUS,C
	rlcf	A48+1,f
	rlcf	A48+2,f
	rlcf	A48+3,f
	movff	A48+2,ACC2_D+2
	movff	A48+3,ACC2_D+3
	movlw	b'00111111'
	andwf	ACC2_D+1,f
	movlw	b'00111111'
	andwf	ACC2_D+3,f
	return

; subroutine CALC_STOPF - calc. stop freq.
;	input	FREQ_D STEP_W
;	output	ACC2_D
;	affect	ACC2_D
CALC_STOPF:
	movlw	d'100'		;100 step
	mulwf	STEP_W		;100 * STEP_W
	movff	PRODL,ACC2_D
	movff	PRODH,ACC2_D+1
	clrf	ACC2_D+2
	clrf	ACC2_D+3
	movlw	d'100'
	mulwf	STEP_W+1
	movf	PRODL,w
	addwf	ACC2_D+1,f
	movf	PRODH,w
	addwfc	ACC2_D+2,f
	btfsc	STATUS,C
	incf	ACC2_D+3
;
	movf	START_D,w	;+START_D
	addwf	ACC2_D,f
	movf	START_D+1,w
	addwfc	ACC2_D+1,f
	movf	START_D+2,w
	addwfc	ACC2_D+2,f
	movf	START_D+3,w
	addwfc	ACC2_D+3,f
	return

; subroutine DISP_START - display start freq.
;	input	START_D = freq.
;	affect	ACC2_D,ACC0_W,TMP0,TMP1,TMP2,PX,PY
DISP_START:
	movff	START_D,ACC2_D
	movff	START_D+1,ACC2_D+1
	movff	START_D+2,ACC2_D+2
	movff	START_D+3,ACC2_D+3
	goto	DFREQ1

; subroutine DISP_FREQ - display freq.data
;	input	FREQ_D = freq.
;	affect	ACC2_D,ACC0_W,TMP0,TMP1,TMP2,PX,PY
DISP_FREQ:
	movff	FREQ_D,ACC2_D
	movff	FREQ_D+1,ACC2_D+1
	movff	FREQ_D+2,ACC2_D+2
	movff	FREQ_D+3,ACC2_D+3
DFREQ1:	call	PUTDEC
	movff	SBUF+2,SBUF
	movff	SBUF+3,SBUF+1
	movff	SBUF+4,SBUF+3
	movff	SBUF+5,SBUF+4
	movff	SBUF+6,SBUF+5
	movlw	'.'
	movwf	SBUF+2
	movwf	SBUF+6
	call	PSTRING_LCD
	movlw	LOW M_UNIT_MHz
	movwf	ACC0_W
	movlw	HIGH M_UNIT_MHz
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	return

; subroutine DISP_STEP - display freq.step
;	input	ACC0_W = freq.step
;	affect	ACC2_D,ACC0_W,TMP0,TMP1,TMP2,PX,PY
DISP_STEP:
	movff	ACC0_W,ACC2_D
	movff	ACC0_W+1,ACC2_D+1
	clrf	ACC2_D+2
	clrf	ACC2_D+3
; check KHz
	movlw	LOW d'1000'
	subwf	ACC0_W,w
	movlw	HIGH d'1000'
	btfss	STATUS,C
	movlw	(HIGH d'1000')+1
	subwf	ACC0_W+1,w
	btfsc	STATUS,C
	goto	DFRK
; Hz
	call	PUTDEC
	movff	SBUF+7,SBUF
	movff	SBUF+8,SBUF+1
	movff	SBUF+9,SBUF+2
	clrf	SBUF+3
	call	PSTRING_LCD
	movlw	LOW M_UNIT_Hz
	movwf	ACC0_W
	movlw	HIGH M_UNIT_Hz
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	return
; KHz
DFRK:	call	PUTDEC
	movff	SBUF+5,SBUF
	movff	SBUF+6,SBUF+1
	movlw	'.'
	movwf	SBUF+2
	movff	SBUF+7,SBUF+3
	movff	SBUF+8,SBUF+4
	movff	SBUF+9,SBUF+5
	clrf	SBUF+6
	call	PSTRING_LCD
	movlw	LOW M_UNIT_KHz
	movwf	ACC0_W
	movlw	HIGH M_UNIT_KHz
	movwf	ACC0_W+1
	call	PSTRING_LCD_FLASH
	return
	
; subroutine LCD_CLR_EOL
;	input	PX,PY = current pos.
;	output	nothing
;	affect	TMP0,TMP1,ACC0_W,PX
LCD_CLR_EOL:
	movlw	d'125'
	subwf	PX,w
	btfss	STATUS,C
	return
	movlw	' '
	call	PUTC_LCD
	goto	LCD_CLR_EOL

; subroutine SET_STEP
;	input	STEPPTR
;	output	STEP_W
;	affect	
SET_STEP:
	movlw	b'00001111'	;check pointer
	andwf	STEPPTR,f
	movf	STEPPTR,w
	addwf	STEPPTR,w	;*2
	addlw	LOW TFSTEP
	movwf	TBLPTRL
	movlw	HIGH TFSTEP
	movwf	TBLPTRH
	clrf	TBLPTRU
	TBLRD*+
	movf	TABLAT,w
	movwf	STEP_W
	TBLRD*+
	movf	TABLAT,w
	movwf	STEP_W+1
	return

; subroutine ADC - AD channel #0
;	input	W=channel(0..3)
;	output	ACC0_W=data(10bit)
;	affect
ADC:	movwf	ACC0_W
	rlcf	ACC0_W,f
	rlcf	ACC0_W,f
	movlw	b'00001100'
	andwf	ACC0_W,f
	movlw	b'00001100'
	movwf	ADCON1
	movlw	b'00000001'
	iorwf	ACC0_W,w
;		  ..00**..	;channel #0..#2
;		  ......*.	;GO/DONE flag
;		  .......1	;ADC ON
	movwf	ADCON0
	movlw	b'10111110'
;		  1.......	;right justified
;		  ..111...	;20Tad acquisition time
;		  .....110	;fosc/64 ADC clock
	movwf	ADCON2
;
	movlw	d'30'
	movwf	ACC0_W
ADC1:	clrwdt
	decfsz	ACC0_W,f
	goto	ADC1

	clrwdt
	bsf	ADCON0,GO
ADC2:	btfsc	ADCON0,GO	;wait AD complete
	goto	ADC2
	movff	ADRESL,ACC0_W
	movff	ADRESH,ACC0_W+1
	return




	org	h'3000'		;table area
; frequency step table
TFSTEP:
	dw	d'1'		;# is 15
	dw	d'2'
	dw	d'5'
	dw	d'10'
	dw	d'20'
	dw	d'50'
	dw	d'100'
	dw	d'200'
	dw	d'500'
	dw	d'1000'
	dw	d'2000'
	dw	d'5000'
	dw	d'10000'
	dw	d'20000'
	dw	d'50000'
	dw	d'50000'		;scratch pad

FONT5x7:
	dw	b'0000000000000000'
	dw	b'0000000000000000'
	dw	b'0000000000000000'
	dw	b'0000000000000000'
	dw	b'0000000001011111'
	dw	b'0000000000000000'
	dw	b'0000001100000000'
	dw	b'0000001100000000'
	dw	b'0000000000000000'
	dw	b'0111111100100010'
	dw	b'0111111100100010'
	dw	b'0000000000100010'
	dw	b'0010101000100100'
	dw	b'0010101001111111'
	dw	b'0000000000010010'
	dw	b'0001001100100011'
	dw	b'0000010000001000'
	dw	b'0110000001100010'
	dw	b'0100010100111010'
	dw	b'0011000001001010'
	dw	b'0100000000101000'
	dw	b'0000000000000000'
	dw	b'0000001100000100'
	dw	b'0000000000000000'
	dw	b'0001110000000000'
	dw	b'0100000100100010'
	dw	b'0000000000000000'
	dw	b'0100000100000000'
	dw	b'0001110000100010'
	dw	b'0000000000000000'
	dw	b'0001110000101010'
	dw	b'0001110001111111'
	dw	b'0000000000101010'
	dw	b'0000100000001000'
	dw	b'0000100001111111'
	dw	b'0000000000001000'
	dw	b'0000000000000000'
	dw	b'0110000010100000'
	dw	b'0000000000000000'
	dw	b'0000100000001000'
	dw	b'0000100000001000'
	dw	b'0000000000001000'
	dw	b'0000000000000000'
	dw	b'0110000001100000'
	dw	b'0000000000000000'
	dw	b'0001000000100000'
	dw	b'0000010000001000'
	dw	b'0000000000000010'
	dw	b'0101000100111110'
	dw	b'0100010101001001'
	dw	b'0000000000111110'
	dw	b'0000001000000000'
	dw	b'0000000001111111'
	dw	b'0000000000000000'
	dw	b'0110000101000010'
	dw	b'0100100101010001'
	dw	b'0000000001000110'
	dw	b'0100000100100010'
	dw	b'0100100101001001'
	dw	b'0000000000110110'
	dw	b'0001010000011000'
	dw	b'0111111100010010'
	dw	b'0000000000010000'
	dw	b'0100100100101111'
	dw	b'0100100101001001'
	dw	b'0000000000110001'
	dw	b'0100100100111110'
	dw	b'0100100101001001'
	dw	b'0000000000110010'
	dw	b'0000000100000011'
	dw	b'0000010101111001'
	dw	b'0000000000000011'
	dw	b'0100100100110110'
	dw	b'0100100101001001'
	dw	b'0000000000110110'
	dw	b'0100100100000110'
	dw	b'0010100101001001'
	dw	b'0000000000011110'
	dw	b'0000000000000000'
	dw	b'0011011000110110'
	dw	b'0000000000000000'
	dw	b'0000000000000000'
	dw	b'0011011001010110'
	dw	b'0000000000000000'
	dw	b'0001010000001000'
	dw	b'0100000100100010'
	dw	b'0000000000000000'
	dw	b'0001010000010100'
	dw	b'0001010000010100'
	dw	b'0000000000010100'
	dw	b'0100000100000000'
	dw	b'0001010000100010'
	dw	b'0000000000001000'
	dw	b'0000000100000010'
	dw	b'0000100101010001'
	dw	b'0000000000000110'
	dw	b'0100100100110010'
	dw	b'0100000101111001'
	dw	b'0000000000111110'
	dw	b'0001001001111100'
	dw	b'0001001000010001'
	dw	b'0000000001111100'
	dw	b'0100100101111111'
	dw	b'0100100101001001'
	dw	b'0010000000110110'
	dw	b'0100000100111110'
	dw	b'0100000101000001'
	dw	b'0000000000100010'
	dw	b'0100000101111111'
	dw	b'0010001001000001'
	dw	b'0000000000011100'
	dw	b'0100100101111111'
	dw	b'0100100101001001'
	dw	b'0000000001001001'
	dw	b'0000100101111111'
	dw	b'0000100100001001'
	dw	b'0000000000000001'
	dw	b'0100000100111110'
	dw	b'0010100101000001'
	dw	b'0000000001111010'
	dw	b'0000100001111111'
	dw	b'0000100000001000'
	dw	b'0000000001111111'
	dw	b'0100000100000000'
	dw	b'0100000101111111'
	dw	b'0000000000000000'
	dw	b'0100000000100000'
	dw	b'0011111101000001'
	dw	b'0000000000000001'
	dw	b'0000100001111111'
	dw	b'0010001000010100'
	dw	b'0000000001000001'
	dw	b'0100000001111111'
	dw	b'0100000001000000'
	dw	b'0000000001000000'
	dw	b'0000001001111111'
	dw	b'0000001000001100'
	dw	b'0000000001111111'
	dw	b'0000010001111111'
	dw	b'0001000000001000'
	dw	b'0000000001111111'
	dw	b'0100000100111110'
	dw	b'0100000101000001'
	dw	b'0000000000111110'
	dw	b'0000100101111111'
	dw	b'0000100100001001'
	dw	b'0000000000000110'
	dw	b'0100000100111110'
	dw	b'0010000101010001'
	dw	b'0000000001011110'
	dw	b'0000100101111111'
	dw	b'0010100100011001'
	dw	b'0000000001000110'
	dw	b'0100100100100110'
	dw	b'0100100101001001'
	dw	b'0000000000110010'
	dw	b'0000000100000001'
	dw	b'0000000101111111'
	dw	b'0000000000000001'
	dw	b'0100000000111111'
	dw	b'0100000001000000'
	dw	b'0000000000111111'
	dw	b'0010000000011111'
	dw	b'0010000001000000'
	dw	b'0000000000011111'
	dw	b'0010000001111111'
	dw	b'0010000000011000'
	dw	b'0000000001111111'
	dw	b'0001010001100011'
	dw	b'0001010000001000'
	dw	b'0000000001100011'
	dw	b'0000010000000011'
	dw	b'0000010001111000'
	dw	b'0000100000000011'
	dw	b'0101000101100001'
	dw	b'0100010101001001'
	dw	b'0000000001000011'
	dw	b'0000000000000000'
	dw	b'0100000101111111'
	dw	b'0000000001000001'
	dw	b'0000010000000010'
	dw	b'0001000000001000'
	dw	b'0000000000100000'
	dw	b'0100000100000000'
	dw	b'0111111101000001'
	dw	b'0000000000000000'
	dw	b'0000001000000100'
	dw	b'0000001000000001'
	dw	b'0000000000000100'
	dw	b'0100000001000000'
	dw	b'0100000001000000'
	dw	b'0000000001000000'
	dw	b'0000001100000000'
	dw	b'0000000000000100'
	dw	b'0000000000000000'
	dw	b'0101010000100000'
	dw	b'0011100001010100'
	dw	b'0000000001000000'
	dw	b'0100100001111111'
	dw	b'0100100001001000'
	dw	b'0000000000110000'
	dw	b'0100010000111000'
	dw	b'0100010001000100'
	dw	b'0000000000101000'
	dw	b'0100100000110000'
	dw	b'0010100001001000'
	dw	b'0000000001111111'
	dw	b'0101010000111000'
	dw	b'0101010001010100'
	dw	b'0010000001011000'
	dw	b'0000100000000000'
	dw	b'0000100101111110'
	dw	b'0000000000001001'
	dw	b'1010010010011000'
	dw	b'1010010010100100'
	dw	b'0000000001111000'
	dw	b'0000100001111111'
	dw	b'0000100000001000'
	dw	b'0000000001110000'
	dw	b'0100000000000000'
	dw	b'0100000001111101'
	dw	b'0000000000000000'
	dw	b'1000000001000000'
	dw	b'0111110110000000'
	dw	b'0000000000000000'
	dw	b'0010000001111111'
	dw	b'0010100000010000'
	dw	b'0000000001000100'
	dw	b'0000000100000000'
	dw	b'0000000001111111'
	dw	b'0000000000000000'
	dw	b'0000100001111000'
	dw	b'0000100001110000'
	dw	b'0000000001110000'
	dw	b'0000100001111000'
	dw	b'0000100000001000'
	dw	b'0000000001110000'
	dw	b'0100100000110000'
	dw	b'0100100001001000'
	dw	b'0000000000110000'
	dw	b'0010100011111000'
	dw	b'0010100000101000'
	dw	b'0000000000010000'
	dw	b'0010100000010000'
	dw	b'0010100000101000'
	dw	b'0000000011111000'
	dw	b'0111110000000000'
	dw	b'0000100000010000'
	dw	b'0000010000001000'
	dw	b'0101010001001000'
	dw	b'0101010001010100'
	dw	b'0000000000100100'
	dw	b'0000100000001000'
	dw	b'0100100000111100'
	dw	b'0000000001001000'
	dw	b'0100000000111100'
	dw	b'0011110001000000'
	dw	b'0000000001000000'
	dw	b'0010000000011000'
	dw	b'0010000001000000'
	dw	b'0000000000011000'
	dw	b'0010000001111000'
	dw	b'0010000000010000'
	dw	b'0000000001111000'
	dw	b'0010100001000100'
	dw	b'0010100000010000'
	dw	b'0000000001000100'
	dw	b'0101000010001000'
	dw	b'0001000000100000'
	dw	b'0000000000001000'
	dw	b'0110010001000100'
	dw	b'0100110001010100'
	dw	b'0000000001000100'
	dw	b'0000000000000000'
	dw	b'0011011000001000'
	dw	b'0000000001000001'
	dw	b'0000000000000000'
	dw	b'0000000001110111'
	dw	b'0000000000000000'
	dw	b'0011011001000001'
	dw	b'0000000000001000'
	dw	b'0000000000000000'
	dw	b'0000000100000010'
	dw	b'0000010000000010'
	dw	b'0000000000000010'
	dw	b'0111110010000000'
	dw	b'0100000001000000'
	dw	b'0100000000111100'
;
; Messages
M_MODE_SG:
	dw	'S'+'G'*h'100'
	dw	' '+'M'*h'100'
	dw	'o'+'d'*h'100'
	dw	'e'
	dw	0
M_MODE_SWEEP:
	dw	'S'+'w'*h'100'
	dw	'e'+'e'*h'100'
	dw	'p'+' '*h'100'
	dw	'M'+'o'*h'100'
	dw	'd'+'e'*h'100'
	dw	0
M_MSG_FREQ:
	dw	'F'+'r'*h'100'
	dw	'e'+'q'*h'100'
	dw	0
M_MSG_WAVE:
	dw	'W'+'a'*h'100'
	dw	'v'+'e'*h'100'
	dw	0
M_MSG_MOD:
	dw	'M'+'o'*h'100'
	dw	'd'+'u'*h'100'
	dw	0
M_MSG_MDFR:
	dw	'M'+'d'*h'100'
	dw	'F'+'r'*h'100'
	dw	0
M_MSG_DEV:
	dw	'D'+'e'*h'100'
	dw	'v'+' '*h'100' 
	dw	0
M_MSG_STEP:
	dw	'S'+'t'*h'100'
	dw	'e'+'p'*h'100'
	dw	0
M_MSG_START:
	dw	'S'+'t'*h'100'
	dw	'r'+'t'*h'100'
	dw	0
M_MSG_END:
	dw	'E'+'n'*h'100'
	dw	'd'+' '*h'100' 
	dw	0
M_MSG_DIAL:
	dw	'D'+'i'*h'100'
	dw	'a'+'l'*h'100'
	dw	0
M_MSG_OUT:
	dw	'O'+'u'*h'100'
	dw	't'
	dw	0
M_MSG_ON:
	dw	'O'+'N'*h'100'
	dw	0
M_MSG_OFF:
	dw	'O'+'f'*h'100'
	dw	'f'
	dw	0
M_MSG_DISP:
	dw	'D'+'i'*h'100'
	dw	's'+'p'*h'100'
	dw	0
M_MSG_GAIN:
	dw	'G'+'a'*h'100'
	dw	'i'+'n'*h'100'
	dw	' '
	dw	0
M_MSG_PHASE:
	dw	'P'+'h'*h'100'
	dw	'a'+'s'*h'100'
	dw	'e'
	dw	0
M_UNIT_MHz:
	dw	'M'+'H'*h'100'
	dw	'z'
	dw	0
M_UNIT_KHz:
	dw	'K'+'H'*h'100'
	dw	'z'
	dw	0
M_UNIT_Hz:
	dw	'H'+'z'*h'100'
	dw	0

M_WAVE_SIN:
	dw	'S'+'i'*h'100'
	dw	'n'
	dw	0
M_WAVE_TRI:
	dw	'T'+'r'*h'100'
	dw	'i'
	dw	0
M_MOD_CW:
	dw	'C'+'W'*h'100'
	dw	' '
	dw	0
M_MOD_OOK:
	dw	'O'+'O'*h'100'
	dw	'K'
	dw	0
M_MOD_FSK:
	dw	'F'+'S'*h'100'
	dw	'K'
	dw	0

	org	h'3400'
DB_TABLE:
	dw	d'0'	;deci-bell*100 table
	dw	d'1'
	dw	d'2'
	dw	d'3'
	dw	d'3'
	dw	d'4'
	dw	d'5'
	dw	d'6'
	dw	d'7'
	dw	d'8'
	dw	d'9'
	dw	d'9'
	dw	d'10'
	dw	d'11'
	dw	d'12'
	dw	d'13'
	dw	d'14'
	dw	d'15'
	dw	d'15'
	dw	d'16'
	dw	d'17'
	dw	d'18'
	dw	d'19'
	dw	d'20'
	dw	d'21'
	dw	d'21'
	dw	d'22'
	dw	d'23'
	dw	d'24'
	dw	d'25'
	dw	d'26'
	dw	d'27'
	dw	d'28'
	dw	d'28'
	dw	d'29'
	dw	d'30'
	dw	d'31'
	dw	d'32'
	dw	d'33'
	dw	d'34'
	dw	d'35'
	dw	d'35'
	dw	d'36'
	dw	d'37'
	dw	d'38'
	dw	d'39'
	dw	d'40'
	dw	d'41'
	dw	d'42'
	dw	d'43'
	dw	d'43'
	dw	d'44'
	dw	d'45'
	dw	d'46'
	dw	d'47'
	dw	d'48'
	dw	d'49'
	dw	d'50'
	dw	d'51'
	dw	d'52'
	dw	d'52'
	dw	d'53'
	dw	d'54'
	dw	d'55'
	dw	d'56'
	dw	d'57'
	dw	d'58'
	dw	d'59'
	dw	d'60'
	dw	d'61'
	dw	d'62'
	dw	d'62'
	dw	d'63'
	dw	d'64'
	dw	d'65'
	dw	d'66'
	dw	d'67'
	dw	d'68'
	dw	d'69'
	dw	d'70'
	dw	d'71'
	dw	d'72'
	dw	d'72'
	dw	d'73'
	dw	d'74'
	dw	d'75'
	dw	d'76'
	dw	d'77'
	dw	d'78'
	dw	d'79'
	dw	d'80'
	dw	d'81'
	dw	d'82'
	dw	d'83'
	dw	d'84'
	dw	d'85'
	dw	d'86'
	dw	d'86'
	dw	d'87'
	dw	d'88'
	dw	d'89'
	dw	d'90'
	dw	d'91'
	dw	d'92'
	dw	d'93'
	dw	d'94'
	dw	d'95'
	dw	d'96'
	dw	d'97'
	dw	d'98'
	dw	d'99'
	dw	d'100'
	dw	d'101'
	dw	d'102'
	dw	d'103'
	dw	d'103'
	dw	d'104'
	dw	d'105'
	dw	d'106'
	dw	d'107'
	dw	d'108'
	dw	d'109'
	dw	d'110'
	dw	d'111'
	dw	d'112'
	dw	d'113'
	dw	d'114'
	dw	d'115'
	dw	d'116'
	dw	d'117'
	dw	d'118'
	dw	d'119'
	dw	d'120'
	dw	d'121'
	dw	d'122'
	dw	d'123'
	dw	d'124'
	dw	d'125'
	dw	d'126'
	dw	d'127'
	dw	d'128'
	dw	d'129'
	dw	d'130'
	dw	d'131'
	dw	d'132'
	dw	d'133'
	dw	d'134'
	dw	d'135'
	dw	d'136'
	dw	d'137'
	dw	d'138'
	dw	d'139'
	dw	d'140'
	dw	d'141'
	dw	d'142'
	dw	d'143'
	dw	d'144'
	dw	d'145'
	dw	d'146'
	dw	d'147'
	dw	d'148'
	dw	d'149'
	dw	d'150'
	dw	d'151'
	dw	d'152'
	dw	d'153'
	dw	d'154'
	dw	d'155'
	dw	d'156'
	dw	d'157'
	dw	d'158'
	dw	d'159'
	dw	d'160'
	dw	d'161'
	dw	d'162'
	dw	d'163'
	dw	d'164'
	dw	d'165'
	dw	d'166'
	dw	d'167'
	dw	d'168'
	dw	d'169'
	dw	d'170'
	dw	d'171'
	dw	d'172'
	dw	d'173'
	dw	d'174'
	dw	d'175'
	dw	d'176'
	dw	d'177'
	dw	d'178'
	dw	d'179'
	dw	d'180'
	dw	d'181'
	dw	d'182'
	dw	d'183'
	dw	d'185'
	dw	d'186'
	dw	d'187'
	dw	d'188'
	dw	d'189'
	dw	d'190'
	dw	d'191'
	dw	d'192'
	dw	d'193'
	dw	d'194'
	dw	d'195'
	dw	d'196'
	dw	d'197'
	dw	d'198'
	dw	d'199'
	dw	d'200'
	dw	d'201'
	dw	d'203'
	dw	d'204'
	dw	d'205'
	dw	d'206'
	dw	d'207'
	dw	d'208'
	dw	d'209'
	dw	d'210'
	dw	d'211'
	dw	d'212'
	dw	d'213'
	dw	d'214'
	dw	d'216'
	dw	d'217'
	dw	d'218'
	dw	d'219'
	dw	d'220'
	dw	d'221'
	dw	d'222'
	dw	d'223'
	dw	d'224'
	dw	d'225'
	dw	d'226'
	dw	d'228'
	dw	d'229'
	dw	d'230'
	dw	d'231'
	dw	d'232'
	dw	d'233'
	dw	d'234'
	dw	d'235'
	dw	d'236'
	dw	d'238'
	dw	d'239'
	dw	d'240'
	dw	d'241'
	dw	d'242'
	dw	d'243'
	dw	d'244'
	dw	d'245'
	dw	d'246'
	dw	d'248'
	dw	d'249'
	dw	d'250'
	dw	d'251'
	dw	d'252'
	dw	d'253'
	dw	d'254'
	dw	d'256'
	dw	d'257'
	dw	d'258'
	dw	d'259'
	dw	d'260'
	dw	d'261'
	dw	d'262'
	dw	d'264'
	dw	d'265'
	dw	d'266'
	dw	d'267'
	dw	d'268'
	dw	d'269'
	dw	d'270'
	dw	d'272'
	dw	d'273'
	dw	d'274'
	dw	d'275'
	dw	d'276'
	dw	d'277'
	dw	d'279'
	dw	d'280'
	dw	d'281'
	dw	d'282'
	dw	d'283'
	dw	d'284'
	dw	d'286'
	dw	d'287'
	dw	d'288'
	dw	d'289'
	dw	d'290'
	dw	d'292'
	dw	d'293'
	dw	d'294'
	dw	d'295'
	dw	d'296'
	dw	d'298'
	dw	d'299'
	dw	d'300'
	dw	d'301'
	dw	d'302'
	dw	d'304'
	dw	d'305'
	dw	d'306'
	dw	d'307'
	dw	d'308'
	dw	d'310'
	dw	d'311'
	dw	d'312'
	dw	d'313'
	dw	d'314'
	dw	d'316'
	dw	d'317'
	dw	d'318'
	dw	d'319'
	dw	d'321'
	dw	d'322'
	dw	d'323'
	dw	d'324'
	dw	d'325'
	dw	d'327'
	dw	d'328'
	dw	d'329'
	dw	d'330'
	dw	d'332'
	dw	d'333'
	dw	d'334'
	dw	d'335'
	dw	d'337'
	dw	d'338'
	dw	d'339'
	dw	d'340'
	dw	d'342'
	dw	d'343'
	dw	d'344'
	dw	d'345'
	dw	d'347'
	dw	d'348'
	dw	d'349'
	dw	d'350'
	dw	d'352'
	dw	d'353'
	dw	d'354'
	dw	d'356'
	dw	d'357'
	dw	d'358'
	dw	d'359'
	dw	d'361'
	dw	d'362'
	dw	d'363'
	dw	d'365'
	dw	d'366'
	dw	d'367'
	dw	d'368'
	dw	d'370'
	dw	d'371'
	dw	d'372'
	dw	d'374'
	dw	d'375'
	dw	d'376'
	dw	d'378'
	dw	d'379'
	dw	d'380'
	dw	d'382'
	dw	d'383'
	dw	d'384'
	dw	d'385'
	dw	d'387'
	dw	d'388'
	dw	d'389'
	dw	d'391'
	dw	d'392'
	dw	d'393'
	dw	d'395'
	dw	d'396'
	dw	d'397'
	dw	d'399'
	dw	d'400'
	dw	d'401'
	dw	d'403'
	dw	d'404'
	dw	d'406'
	dw	d'407'
	dw	d'408'
	dw	d'410'
	dw	d'411'
	dw	d'412'
	dw	d'414'
	dw	d'415'
	dw	d'416'
	dw	d'418'
	dw	d'419'
	dw	d'421'
	dw	d'422'
	dw	d'423'
	dw	d'425'
	dw	d'426'
	dw	d'427'
	dw	d'429'
	dw	d'430'
	dw	d'432'
	dw	d'433'
	dw	d'434'
	dw	d'436'
	dw	d'437'
	dw	d'439'
	dw	d'440'
	dw	d'441'
	dw	d'443'
	dw	d'444'
	dw	d'446'
	dw	d'447'
	dw	d'449'
	dw	d'450'
	dw	d'451'
	dw	d'453'
	dw	d'454'
	dw	d'456'
	dw	d'457'
	dw	d'459'
	dw	d'460'
	dw	d'461'
	dw	d'463'
	dw	d'464'
	dw	d'466'
	dw	d'467'
	dw	d'469'
	dw	d'470'
	dw	d'472'
	dw	d'473'
	dw	d'474'
	dw	d'476'
	dw	d'477'
	dw	d'479'
	dw	d'480'
	dw	d'482'
	dw	d'483'
	dw	d'485'
	dw	d'486'
	dw	d'488'
	dw	d'489'
	dw	d'491'
	dw	d'492'
	dw	d'494'
	dw	d'495'
	dw	d'497'
	dw	d'498'
	dw	d'500'
	dw	d'501'
	dw	d'503'
	dw	d'504'
	dw	d'506'
	dw	d'507'
	dw	d'509'
	dw	d'510'
	dw	d'512'
	dw	d'513'
	dw	d'515'
	dw	d'517'
	dw	d'518'
	dw	d'520'
	dw	d'521'
	dw	d'523'
	dw	d'524'
	dw	d'526'
	dw	d'527'
	dw	d'529'
	dw	d'530'
	dw	d'532'
	dw	d'534'
	dw	d'535'
	dw	d'537'
	dw	d'538'
	dw	d'540'
	dw	d'541'
	dw	d'543'
	dw	d'545'
	dw	d'546'
	dw	d'548'
	dw	d'549'
	dw	d'551'
	dw	d'553'
	dw	d'554'
	dw	d'556'
	dw	d'557'
	dw	d'559'
	dw	d'561'
	dw	d'562'
	dw	d'564'
	dw	d'566'
	dw	d'567'
	dw	d'569'
	dw	d'570'
	dw	d'572'
	dw	d'574'
	dw	d'575'
	dw	d'577'
	dw	d'579'
	dw	d'580'
	dw	d'582'
	dw	d'584'
	dw	d'585'
	dw	d'587'
	dw	d'589'
	dw	d'590'
	dw	d'592'
	dw	d'594'
	dw	d'595'
	dw	d'597'
	dw	d'599'
	dw	d'600'
	dw	d'602'
	dw	d'604'
	dw	d'605'
	dw	d'607'
	dw	d'609'
	dw	d'611'
	dw	d'612'
	dw	d'614'
	dw	d'616'
	dw	d'617'
	dw	d'619'
	dw	d'621'
	dw	d'623'
	dw	d'624'
	dw	d'626'
	dw	d'628'
	dw	d'630'
	dw	d'631'
	dw	d'633'
	dw	d'635'
	dw	d'637'
	dw	d'638'
	dw	d'640'
	dw	d'642'
	dw	d'644'
	dw	d'646'
	dw	d'647'
	dw	d'649'
	dw	d'651'
	dw	d'653'
	dw	d'655'
	dw	d'656'
	dw	d'658'
	dw	d'660'
	dw	d'662'
	dw	d'664'
	dw	d'665'
	dw	d'667'
	dw	d'669'
	dw	d'671'
	dw	d'673'
	dw	d'675'
	dw	d'676'
	dw	d'678'
	dw	d'680'
	dw	d'682'
	dw	d'684'
	dw	d'686'
	dw	d'688'
	dw	d'689'
	dw	d'691'
	dw	d'693'
	dw	d'695'
	dw	d'697'
	dw	d'699'
	dw	d'701'
	dw	d'703'
	dw	d'705'
	dw	d'706'
	dw	d'708'
	dw	d'710'
	dw	d'712'
	dw	d'714'
	dw	d'716'
	dw	d'718'
	dw	d'720'
	dw	d'722'
	dw	d'724'
	dw	d'726'
	dw	d'728'
	dw	d'730'
	dw	d'732'
	dw	d'734'
	dw	d'736'
	dw	d'738'
	dw	d'740'
	dw	d'742'
	dw	d'744'
	dw	d'746'
	dw	d'748'
	dw	d'750'
	dw	d'752'
	dw	d'754'
	dw	d'756'
	dw	d'758'
	dw	d'760'
	dw	d'762'
	dw	d'764'
	dw	d'766'
	dw	d'768'
	dw	d'770'
	dw	d'772'
	dw	d'774'
	dw	d'776'
	dw	d'778'
	dw	d'780'
	dw	d'782'
	dw	d'785'
	dw	d'787'
	dw	d'789'
	dw	d'791'
	dw	d'793'
	dw	d'795'
	dw	d'797'
	dw	d'799'
	dw	d'801'
	dw	d'804'
	dw	d'806'
	dw	d'808'
	dw	d'810'
	dw	d'812'
	dw	d'814'
	dw	d'816'
	dw	d'819'
	dw	d'821'
	dw	d'823'
	dw	d'825'
	dw	d'827'
	dw	d'830'
	dw	d'832'
	dw	d'834'
	dw	d'836'
	dw	d'838'
	dw	d'841'
	dw	d'843'
	dw	d'845'
	dw	d'847'
	dw	d'850'
	dw	d'852'
	dw	d'854'
	dw	d'856'
	dw	d'859'
	dw	d'861'
	dw	d'863'
	dw	d'866'
	dw	d'868'
	dw	d'870'
	dw	d'873'
	dw	d'875'
	dw	d'877'
	dw	d'880'
	dw	d'882'
	dw	d'884'
	dw	d'887'
	dw	d'889'
	dw	d'891'
	dw	d'894'
	dw	d'896'
	dw	d'898'
	dw	d'901'
	dw	d'903'
	dw	d'906'
	dw	d'908'
	dw	d'910'
	dw	d'913'
	dw	d'915'
	dw	d'918'
	dw	d'920'
	dw	d'923'
	dw	d'925'
	dw	d'928'
	dw	d'930'
	dw	d'932'
	dw	d'935'
	dw	d'937'
	dw	d'940'
	dw	d'942'
	dw	d'945'
	dw	d'947'
	dw	d'950'
	dw	d'953'
	dw	d'955'
	dw	d'958'
	dw	d'960'
	dw	d'963'
	dw	d'965'
	dw	d'968'
	dw	d'971'
	dw	d'973'
	dw	d'976'
	dw	d'978'
	dw	d'981'
	dw	d'984'
	dw	d'986'
	dw	d'989'
	dw	d'992'
	dw	d'994'
	dw	d'997'
	dw	d'1000'
	dw	d'1002'
	dw	d'1005'
	dw	d'1008'
	dw	d'1010'
	dw	d'1013'
	dw	d'1016'
	dw	d'1018'
	dw	d'1021'
	dw	d'1024'
	dw	d'1027'
	dw	d'1030'
	dw	d'1032'
	dw	d'1035'
	dw	d'1038'
	dw	d'1041'
	dw	d'1043'
	dw	d'1046'
	dw	d'1049'
	dw	d'1052'
	dw	d'1055'
	dw	d'1058'
	dw	d'1061'
	dw	d'1063'
	dw	d'1066'
	dw	d'1069'
	dw	d'1072'
	dw	d'1075'
	dw	d'1078'
	dw	d'1081'
	dw	d'1084'
	dw	d'1087'
	dw	d'1090'
	dw	d'1093'
	dw	d'1096'
	dw	d'1099'
	dw	d'1102'
	dw	d'1105'
	dw	d'1108'
	dw	d'1111'
	dw	d'1114'
	dw	d'1117'
	dw	d'1120'
	dw	d'1123'
	dw	d'1126'
	dw	d'1129'
	dw	d'1133'
	dw	d'1136'
	dw	d'1139'
	dw	d'1142'
	dw	d'1145'
	dw	d'1148'
	dw	d'1151'
	dw	d'1155'
	dw	d'1158'
	dw	d'1161'
	dw	d'1164'
	dw	d'1168'
	dw	d'1171'
	dw	d'1174'
	dw	d'1177'
	dw	d'1181'
	dw	d'1184'
	dw	d'1187'
	dw	d'1191'
	dw	d'1194'
	dw	d'1197'
	dw	d'1201'
	dw	d'1204'
	dw	d'1208'
	dw	d'1211'
	dw	d'1214'
	dw	d'1218'
	dw	d'1221'
	dw	d'1225'
	dw	d'1228'
	dw	d'1232'
	dw	d'1235'
	dw	d'1239'
	dw	d'1242'
	dw	d'1246'
	dw	d'1249'
	dw	d'1253'
	dw	d'1257'
	dw	d'1260'
	dw	d'1264'
	dw	d'1267'
	dw	d'1271'
	dw	d'1275'
	dw	d'1278'
	dw	d'1282'
	dw	d'1286'
	dw	d'1290'
	dw	d'1293'
	dw	d'1297'
	dw	d'1301'
	dw	d'1305'
	dw	d'1309'
	dw	d'1312'
	dw	d'1316'
	dw	d'1320'
	dw	d'1324'
	dw	d'1328'
	dw	d'1332'
	dw	d'1336'
	dw	d'1340'
	dw	d'1344'
	dw	d'1348'
	dw	d'1352'
	dw	d'1356'
	dw	d'1360'
	dw	d'1364'
	dw	d'1368'
	dw	d'1372'
	dw	d'1376'
	dw	d'1380'
	dw	d'1384'
	dw	d'1389'
	dw	d'1393'
	dw	d'1397'
	dw	d'1401'
	dw	d'1406'
	dw	d'1410'
	dw	d'1414'
	dw	d'1419'
	dw	d'1423'
	dw	d'1427'
	dw	d'1432'
	dw	d'1436'
	dw	d'1441'
	dw	d'1445'
	dw	d'1449'
	dw	d'1454'
	dw	d'1459'
	dw	d'1463'
	dw	d'1468'
	dw	d'1472'
	dw	d'1477'
	dw	d'1482'
	dw	d'1486'
	dw	d'1491'
	dw	d'1496'
	dw	d'1500'
	dw	d'1505'
	dw	d'1510'
	dw	d'1515'
	dw	d'1520'
	dw	d'1525'
	dw	d'1530'
	dw	d'1535'
	dw	d'1540'
	dw	d'1545'
	dw	d'1550'
	dw	d'1555'
	dw	d'1560'
	dw	d'1565'
	dw	d'1570'
	dw	d'1575'
	dw	d'1580'
	dw	d'1586'
	dw	d'1591'
	dw	d'1596'
	dw	d'1602'
	dw	d'1607'
	dw	d'1612'
	dw	d'1618'
	dw	d'1623'
	dw	d'1629'
	dw	d'1634'
	dw	d'1640'
	dw	d'1646'
	dw	d'1651'
	dw	d'1657'
	dw	d'1663'
	dw	d'1668'
	dw	d'1674'
	dw	d'1680'
	dw	d'1686'
	dw	d'1692'
	dw	d'1698'
	dw	d'1704'
	dw	d'1710'
	dw	d'1716'
	dw	d'1722'
	dw	d'1728'
	dw	d'1735'
	dw	d'1741'
	dw	d'1747'
	dw	d'1754'
	dw	d'1760'
	dw	d'1766'
	dw	d'1773'
	dw	d'1779'
	dw	d'1786'
	dw	d'1793'
	dw	d'1799'
	dw	d'1806'
	dw	d'1813'
	dw	d'1820'
	dw	d'1827'
	dw	d'1834'
	dw	d'1841'
	dw	d'1848'
	dw	d'1855'
	dw	d'1862'
	dw	d'1870'
	dw	d'1877'
	dw	d'1884'
	dw	d'1892'
	dw	d'1899'
	dw	d'1907'
	dw	d'1914'
	dw	d'1922'
	dw	d'1930'
	dw	d'1938'
	dw	d'1946'
	dw	d'1954'
	dw	d'1962'
	dw	d'1970'
	dw	d'1978'
	dw	d'1987'
	dw	d'1995'
	dw	d'2003'
	dw	d'2012'
	dw	d'2021'
	dw	d'2029'
	dw	d'2038'
	dw	d'2047'
	dw	d'2056'
	dw	d'2065'
	dw	d'2074'
	dw	d'2084'
	dw	d'2093'
	dw	d'2103'
	dw	d'2112'
	dw	d'2122'
	dw	d'2132'
	dw	d'2142'
	dw	d'2152'
	dw	d'2162'
	dw	d'2172'
	dw	d'2182'
	dw	d'2193'
	dw	d'2204'
	dw	d'2214'
	dw	d'2225'
	dw	d'2236'
	dw	d'2248'
	dw	d'2259'
	dw	d'2270'
	dw	d'2282'
	dw	d'2294'
	dw	d'2306'
	dw	d'2318'
	dw	d'2330'
	dw	d'2343'
	dw	d'2356'
	dw	d'2368'
	dw	d'2382'
	dw	d'2395'
	dw	d'2408'
	dw	d'2422'
	dw	d'2436'
	dw	d'2450'
	dw	d'2464'
	dw	d'2479'
	dw	d'2494'
	dw	d'2509'
	dw	d'2524'
	dw	d'2540'
	dw	d'2556'
	dw	d'2572'
	dw	d'2589'
	dw	d'2605'
	dw	d'2623'
	dw	d'2640'
	dw	d'2658'
	dw	d'2676'
	dw	d'2695'
	dw	d'2714'
	dw	d'2734'
	dw	d'2754'
	dw	d'2774'
	dw	d'2795'
	dw	d'2816'
	dw	d'2838'
	dw	d'2861'
	dw	d'2884'
	dw	d'2908'
	dw	d'2932'
	dw	d'2958'
	dw	d'2984'
	dw	d'3010'
	dw	d'3038'
	dw	d'3066'
	dw	d'3096'
	dw	d'3126'
	dw	d'3158'
	dw	d'3191'
	dw	d'3225'
	dw	d'3260'
	dw	d'3297'
	dw	d'3336'
	dw	d'3376'
	dw	d'3419'
	dw	d'3463'
	dw	d'3510'
	dw	d'3560'
	dw	d'3612'
	dw	d'3668'
	dw	d'3728'
	dw	d'3793'
	dw	d'3862'
	dw	d'3938'
	dw	d'4021'
	dw	d'4112'
	dw	d'4214'
	dw	d'4330'
	dw	d'4464'
	dw	d'4623'
	dw	d'4816'
	dw	d'5066'
	dw	d'5419'
	dw	d'6021'


	end